# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six


class OfferMerchantInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "default_shipping_country": "str",
        "feedback_count": "int",
        "feedback_rating": "float",
        "id": "str",
        "name": "str",
    }

    attribute_map = {
        "default_shipping_country": "DefaultShippingCountry",
        "feedback_count": "FeedbackCount",
        "feedback_rating": "FeedbackRating",
        "id": "Id",
        "name": "Name",
    }

    def __init__(
        self,
        default_shipping_country=None,
        feedback_count=None,
        feedback_rating=None,
        id=None,
        name=None,
    ):  # noqa: E501
        """OfferMerchantInfo - a model defined in Swagger"""  # noqa: E501

        self._default_shipping_country = None
        self._feedback_count = None
        self._feedback_rating = None
        self._id = None
        self._name = None
        self.discriminator = None

        if default_shipping_country is not None:
            self.default_shipping_country = default_shipping_country
        if feedback_count is not None:
            self.feedback_count = feedback_count
        if feedback_rating is not None:
            self.feedback_rating = feedback_rating
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name

    @property
    def default_shipping_country(self):
        """Gets the default_shipping_country of this OfferMerchantInfo.  # noqa: E501


        :return: The default_shipping_country of this OfferMerchantInfo.  # noqa: E501
        :rtype: str
        """
        return self._default_shipping_country

    @default_shipping_country.setter
    def default_shipping_country(self, default_shipping_country):
        """Sets the default_shipping_country of this OfferMerchantInfo.


        :param default_shipping_country: The default_shipping_country of this OfferMerchantInfo.  # noqa: E501
        :type: str
        """

        self._default_shipping_country = default_shipping_country

    @property
    def feedback_count(self):
        """Gets the feedback_count of this OfferMerchantInfo.  # noqa: E501


        :return: The feedback_count of this OfferMerchantInfo.  # noqa: E501
        :rtype: int
        """
        return self._feedback_count

    @feedback_count.setter
    def feedback_count(self, feedback_count):
        """Sets the feedback_count of this OfferMerchantInfo.


        :param feedback_count: The feedback_count of this OfferMerchantInfo.  # noqa: E501
        :type: int
        """

        self._feedback_count = feedback_count

    @property
    def feedback_rating(self):
        """Gets the feedback_rating of this OfferMerchantInfo.  # noqa: E501


        :return: The feedback_rating of this OfferMerchantInfo.  # noqa: E501
        :rtype: float
        """
        return self._feedback_rating

    @feedback_rating.setter
    def feedback_rating(self, feedback_rating):
        """Sets the feedback_rating of this OfferMerchantInfo.


        :param feedback_rating: The feedback_rating of this OfferMerchantInfo.  # noqa: E501
        :type: float
        """

        self._feedback_rating = feedback_rating

    @property
    def id(self):
        """Gets the id of this OfferMerchantInfo.  # noqa: E501


        :return: The id of this OfferMerchantInfo.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OfferMerchantInfo.


        :param id: The id of this OfferMerchantInfo.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this OfferMerchantInfo.  # noqa: E501


        :return: The name of this OfferMerchantInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OfferMerchantInfo.


        :param name: The name of this OfferMerchantInfo.  # noqa: E501
        :type: str
        """

        self._name = name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferMerchantInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferMerchantInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.by_line_info import ByLineInfo  # noqa: F401,E501
from paapi5_python_sdk.models.classifications import Classifications  # noqa: F401,E501
from paapi5_python_sdk.models.content_info import ContentInfo  # noqa: F401,E501
from paapi5_python_sdk.models.content_rating import ContentRating  # noqa: F401,E501
from paapi5_python_sdk.models.external_ids import ExternalIds  # noqa: F401,E501
from paapi5_python_sdk.models.manufacture_info import ManufactureInfo  # noqa: F401,E501
from paapi5_python_sdk.models.multi_valued_attribute import MultiValuedAttribute  # noqa: F401,E501
from paapi5_python_sdk.models.product_info import ProductInfo  # noqa: F401,E501
from paapi5_python_sdk.models.single_string_valued_attribute import (
    SingleStringValuedAttribute,
)  # noqa: F401,E501
from paapi5_python_sdk.models.technical_info import TechnicalInfo  # noqa: F401,E501
from paapi5_python_sdk.models.trade_in_info import TradeInInfo  # noqa: F401,E501


class ItemInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "by_line_info": "ByLineInfo",
        "classifications": "Classifications",
        "content_info": "ContentInfo",
        "content_rating": "ContentRating",
        "external_ids": "ExternalIds",
        "features": "MultiValuedAttribute",
        "manufacture_info": "ManufactureInfo",
        "product_info": "ProductInfo",
        "technical_info": "TechnicalInfo",
        "title": "SingleStringValuedAttribute",
        "trade_in_info": "TradeInInfo",
    }

    attribute_map = {
        "by_line_info": "ByLineInfo",
        "classifications": "Classifications",
        "content_info": "ContentInfo",
        "content_rating": "ContentRating",
        "external_ids": "ExternalIds",
        "features": "Features",
        "manufacture_info": "ManufactureInfo",
        "product_info": "ProductInfo",
        "technical_info": "TechnicalInfo",
        "title": "Title",
        "trade_in_info": "TradeInInfo",
    }

    def __init__(
        self,
        by_line_info=None,
        classifications=None,
        content_info=None,
        content_rating=None,
        external_ids=None,
        features=None,
        manufacture_info=None,
        product_info=None,
        technical_info=None,
        title=None,
        trade_in_info=None,
    ):  # noqa: E501
        """ItemInfo - a model defined in Swagger"""  # noqa: E501

        self._by_line_info = None
        self._classifications = None
        self._content_info = None
        self._content_rating = None
        self._external_ids = None
        self._features = None
        self._manufacture_info = None
        self._product_info = None
        self._technical_info = None
        self._title = None
        self._trade_in_info = None
        self.discriminator = None

        if by_line_info is not None:
            self.by_line_info = by_line_info
        if classifications is not None:
            self.classifications = classifications
        if content_info is not None:
            self.content_info = content_info
        if content_rating is not None:
            self.content_rating = content_rating
        if external_ids is not None:
            self.external_ids = external_ids
        if features is not None:
            self.features = features
        if manufacture_info is not None:
            self.manufacture_info = manufacture_info
        if product_info is not None:
            self.product_info = product_info
        if technical_info is not None:
            self.technical_info = technical_info
        if title is not None:
            self.title = title
        if trade_in_info is not None:
            self.trade_in_info = trade_in_info

    @property
    def by_line_info(self):
        """Gets the by_line_info of this ItemInfo.  # noqa: E501


        :return: The by_line_info of this ItemInfo.  # noqa: E501
        :rtype: ByLineInfo
        """
        return self._by_line_info

    @by_line_info.setter
    def by_line_info(self, by_line_info):
        """Sets the by_line_info of this ItemInfo.


        :param by_line_info: The by_line_info of this ItemInfo.  # noqa: E501
        :type: ByLineInfo
        """

        self._by_line_info = by_line_info

    @property
    def classifications(self):
        """Gets the classifications of this ItemInfo.  # noqa: E501


        :return: The classifications of this ItemInfo.  # noqa: E501
        :rtype: Classifications
        """
        return self._classifications

    @classifications.setter
    def classifications(self, classifications):
        """Sets the classifications of this ItemInfo.


        :param classifications: The classifications of this ItemInfo.  # noqa: E501
        :type: Classifications
        """

        self._classifications = classifications

    @property
    def content_info(self):
        """Gets the content_info of this ItemInfo.  # noqa: E501


        :return: The content_info of this ItemInfo.  # noqa: E501
        :rtype: ContentInfo
        """
        return self._content_info

    @content_info.setter
    def content_info(self, content_info):
        """Sets the content_info of this ItemInfo.


        :param content_info: The content_info of this ItemInfo.  # noqa: E501
        :type: ContentInfo
        """

        self._content_info = content_info

    @property
    def content_rating(self):
        """Gets the content_rating of this ItemInfo.  # noqa: E501


        :return: The content_rating of this ItemInfo.  # noqa: E501
        :rtype: ContentRating
        """
        return self._content_rating

    @content_rating.setter
    def content_rating(self, content_rating):
        """Sets the content_rating of this ItemInfo.


        :param content_rating: The content_rating of this ItemInfo.  # noqa: E501
        :type: ContentRating
        """

        self._content_rating = content_rating

    @property
    def external_ids(self):
        """Gets the external_ids of this ItemInfo.  # noqa: E501


        :return: The external_ids of this ItemInfo.  # noqa: E501
        :rtype: ExternalIds
        """
        return self._external_ids

    @external_ids.setter
    def external_ids(self, external_ids):
        """Sets the external_ids of this ItemInfo.


        :param external_ids: The external_ids of this ItemInfo.  # noqa: E501
        :type: ExternalIds
        """

        self._external_ids = external_ids

    @property
    def features(self):
        """Gets the features of this ItemInfo.  # noqa: E501


        :return: The features of this ItemInfo.  # noqa: E501
        :rtype: MultiValuedAttribute
        """
        return self._features

    @features.setter
    def features(self, features):
        """Sets the features of this ItemInfo.


        :param features: The features of this ItemInfo.  # noqa: E501
        :type: MultiValuedAttribute
        """

        self._features = features

    @property
    def manufacture_info(self):
        """Gets the manufacture_info of this ItemInfo.  # noqa: E501


        :return: The manufacture_info of this ItemInfo.  # noqa: E501
        :rtype: ManufactureInfo
        """
        return self._manufacture_info

    @manufacture_info.setter
    def manufacture_info(self, manufacture_info):
        """Sets the manufacture_info of this ItemInfo.


        :param manufacture_info: The manufacture_info of this ItemInfo.  # noqa: E501
        :type: ManufactureInfo
        """

        self._manufacture_info = manufacture_info

    @property
    def product_info(self):
        """Gets the product_info of this ItemInfo.  # noqa: E501


        :return: The product_info of this ItemInfo.  # noqa: E501
        :rtype: ProductInfo
        """
        return self._product_info

    @product_info.setter
    def product_info(self, product_info):
        """Sets the product_info of this ItemInfo.


        :param product_info: The product_info of this ItemInfo.  # noqa: E501
        :type: ProductInfo
        """

        self._product_info = product_info

    @property
    def technical_info(self):
        """Gets the technical_info of this ItemInfo.  # noqa: E501


        :return: The technical_info of this ItemInfo.  # noqa: E501
        :rtype: TechnicalInfo
        """
        return self._technical_info

    @technical_info.setter
    def technical_info(self, technical_info):
        """Sets the technical_info of this ItemInfo.


        :param technical_info: The technical_info of this ItemInfo.  # noqa: E501
        :type: TechnicalInfo
        """

        self._technical_info = technical_info

    @property
    def title(self):
        """Gets the title of this ItemInfo.  # noqa: E501


        :return: The title of this ItemInfo.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ItemInfo.


        :param title: The title of this ItemInfo.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._title = title

    @property
    def trade_in_info(self):
        """Gets the trade_in_info of this ItemInfo.  # noqa: E501


        :return: The trade_in_info of this ItemInfo.  # noqa: E501
        :rtype: TradeInInfo
        """
        return self._trade_in_info

    @trade_in_info.setter
    def trade_in_info(self, trade_in_info):
        """Sets the trade_in_info of this ItemInfo.


        :param trade_in_info: The trade_in_info of this ItemInfo.  # noqa: E501
        :type: TradeInInfo
        """

        self._trade_in_info = trade_in_info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(ItemInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.offer_listing import OfferListing  # noqa: F401,E501
from paapi5_python_sdk.models.offer_summary import OfferSummary  # noqa: F401,E501


class Offers(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "listings": "list[OfferListing]",
        "summaries": "list[OfferSummary]",
    }

    attribute_map = {"listings": "Listings", "summaries": "Summaries"}

    def __init__(self, listings=None, summaries=None):  # noqa: E501
        """Offers - a model defined in Swagger"""  # noqa: E501

        self._listings = None
        self._summaries = None
        self.discriminator = None

        if listings is not None:
            self.listings = listings
        if summaries is not None:
            self.summaries = summaries

    @property
    def listings(self):
        """Gets the listings of this Offers.  # noqa: E501


        :return: The listings of this Offers.  # noqa: E501
        :rtype: list[OfferListing]
        """
        return self._listings

    @listings.setter
    def listings(self, listings):
        """Sets the listings of this Offers.


        :param listings: The listings of this Offers.  # noqa: E501
        :type: list[OfferListing]
        """

        self._listings = listings

    @property
    def summaries(self):
        """Gets the summaries of this Offers.  # noqa: E501


        :return: The summaries of this Offers.  # noqa: E501
        :rtype: list[OfferSummary]
        """
        return self._summaries

    @summaries.setter
    def summaries(self, summaries):
        """Sets the summaries of this Offers.


        :param summaries: The summaries of this Offers.  # noqa: E501
        :type: list[OfferSummary]
        """

        self._summaries = summaries

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(Offers, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Offers):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.availability import Availability  # noqa: F401,E501
from paapi5_python_sdk.models.condition import Condition  # noqa: F401,E501
from paapi5_python_sdk.models.delivery_flag import DeliveryFlag  # noqa: F401,E501
from paapi5_python_sdk.models.max_price import MaxPrice  # noqa: F401,E501
from paapi5_python_sdk.models.merchant import Merchant  # noqa: F401,E501
from paapi5_python_sdk.models.min_price import MinPrice  # noqa: F401,E501
from paapi5_python_sdk.models.min_reviews_rating import MinReviewsRating  # noqa: F401,E501
from paapi5_python_sdk.models.min_saving_percent import MinSavingPercent  # noqa: F401,E501
from paapi5_python_sdk.models.offer_count import OfferCount  # noqa: F401,E501
from paapi5_python_sdk.models.partner_type import PartnerType  # noqa: F401,E501
from paapi5_python_sdk.models.properties import Properties  # noqa: F401,E501
from paapi5_python_sdk.models.search_items_resource import SearchItemsResource  # noqa: F401,E501
from paapi5_python_sdk.models.sort_by import SortBy  # noqa: F401,E501


class SearchItemsRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "actor": "str",
        "artist": "str",
        "author": "str",
        "availability": "Availability",
        "brand": "str",
        "browse_node_id": "str",
        "condition": "Condition",
        "currency_of_preference": "str",
        "delivery_flags": "list[DeliveryFlag]",
        "item_count": "int",
        "item_page": "int",
        "keywords": "str",
        "languages_of_preference": "list[str]",
        "marketplace": "str",
        "max_price": "MaxPrice",
        "merchant": "Merchant",
        "min_price": "MinPrice",
        "min_reviews_rating": "MinReviewsRating",
        "min_saving_percent": "MinSavingPercent",
        "offer_count": "OfferCount",
        "partner_tag": "str",
        "partner_type": "PartnerType",
        "properties": "Properties",
        "resources": "list[SearchItemsResource]",
        "search_index": "str",
        "sort_by": "SortBy",
        "title": "str",
    }

    attribute_map = {
        "actor": "Actor",
        "artist": "Artist",
        "author": "Author",
        "availability": "Availability",
        "brand": "Brand",
        "browse_node_id": "BrowseNodeId",
        "condition": "Condition",
        "currency_of_preference": "CurrencyOfPreference",
        "delivery_flags": "DeliveryFlags",
        "item_count": "ItemCount",
        "item_page": "ItemPage",
        "keywords": "Keywords",
        "languages_of_preference": "LanguagesOfPreference",
        "marketplace": "Marketplace",
        "max_price": "MaxPrice",
        "merchant": "Merchant",
        "min_price": "MinPrice",
        "min_reviews_rating": "MinReviewsRating",
        "min_saving_percent": "MinSavingPercent",
        "offer_count": "OfferCount",
        "partner_tag": "PartnerTag",
        "partner_type": "PartnerType",
        "properties": "Properties",
        "resources": "Resources",
        "search_index": "SearchIndex",
        "sort_by": "SortBy",
        "title": "Title",
    }

    def __init__(
        self,
        actor=None,
        artist=None,
        author=None,
        availability=None,
        brand=None,
        browse_node_id=None,
        condition=None,
        currency_of_preference=None,
        delivery_flags=None,
        item_count=None,
        item_page=None,
        keywords=None,
        languages_of_preference=None,
        marketplace=None,
        max_price=None,
        merchant=None,
        min_price=None,
        min_reviews_rating=None,
        min_saving_percent=None,
        offer_count=None,
        partner_tag=None,
        partner_type=None,
        properties=None,
        resources=None,
        search_index=None,
        sort_by=None,
        title=None,
    ):  # noqa: E501
        """SearchItemsRequest - a model defined in Swagger"""  # noqa: E501

        self._actor = None
        self._artist = None
        self._author = None
        self._availability = None
        self._brand = None
        self._browse_node_id = None
        self._condition = None
        self._currency_of_preference = None
        self._delivery_flags = None
        self._item_count = None
        self._item_page = None
        self._keywords = None
        self._languages_of_preference = None
        self._marketplace = None
        self._max_price = None
        self._merchant = None
        self._min_price = None
        self._min_reviews_rating = None
        self._min_saving_percent = None
        self._offer_count = None
        self._partner_tag = None
        self._partner_type = None
        self._properties = None
        self._resources = None
        self._search_index = None
        self._sort_by = None
        self._title = None
        self.discriminator = None

        if actor is not None:
            self.actor = actor
        if artist is not None:
            self.artist = artist
        if author is not None:
            self.author = author
        if availability is not None:
            self.availability = availability
        if brand is not None:
            self.brand = brand
        if browse_node_id is not None:
            self.browse_node_id = browse_node_id
        if condition is not None:
            self.condition = condition
        if currency_of_preference is not None:
            self.currency_of_preference = currency_of_preference
        if delivery_flags is not None:
            self.delivery_flags = delivery_flags
        if item_count is not None:
            self.item_count = item_count
        if item_page is not None:
            self.item_page = item_page
        if keywords is not None:
            self.keywords = keywords
        if languages_of_preference is not None:
            self.languages_of_preference = languages_of_preference
        if marketplace is not None:
            self.marketplace = marketplace
        if max_price is not None:
            self.max_price = max_price
        if merchant is not None:
            self.merchant = merchant
        if min_price is not None:
            self.min_price = min_price
        if min_reviews_rating is not None:
            self.min_reviews_rating = min_reviews_rating
        if min_saving_percent is not None:
            self.min_saving_percent = min_saving_percent
        if offer_count is not None:
            self.offer_count = offer_count
        self.partner_tag = partner_tag
        self.partner_type = partner_type
        if properties is not None:
            self.properties = properties
        if resources is not None:
            self.resources = resources
        if search_index is not None:
            self.search_index = search_index
        if sort_by is not None:
            self.sort_by = sort_by
        if title is not None:
            self.title = title

    @property
    def actor(self):
        """Gets the actor of this SearchItemsRequest.  # noqa: E501


        :return: The actor of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._actor

    @actor.setter
    def actor(self, actor):
        """Sets the actor of this SearchItemsRequest.


        :param actor: The actor of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._actor = actor

    @property
    def artist(self):
        """Gets the artist of this SearchItemsRequest.  # noqa: E501


        :return: The artist of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._artist

    @artist.setter
    def artist(self, artist):
        """Sets the artist of this SearchItemsRequest.


        :param artist: The artist of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._artist = artist

    @property
    def author(self):
        """Gets the author of this SearchItemsRequest.  # noqa: E501


        :return: The author of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._author

    @author.setter
    def author(self, author):
        """Sets the author of this SearchItemsRequest.


        :param author: The author of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._author = author

    @property
    def availability(self):
        """Gets the availability of this SearchItemsRequest.  # noqa: E501


        :return: The availability of this SearchItemsRequest.  # noqa: E501
        :rtype: Availability
        """
        return self._availability

    @availability.setter
    def availability(self, availability):
        """Sets the availability of this SearchItemsRequest.


        :param availability: The availability of this SearchItemsRequest.  # noqa: E501
        :type: Availability
        """

        self._availability = availability

    @property
    def brand(self):
        """Gets the brand of this SearchItemsRequest.  # noqa: E501


        :return: The brand of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._brand

    @brand.setter
    def brand(self, brand):
        """Sets the brand of this SearchItemsRequest.


        :param brand: The brand of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._brand = brand

    @property
    def browse_node_id(self):
        """Gets the browse_node_id of this SearchItemsRequest.  # noqa: E501


        :return: The browse_node_id of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._browse_node_id

    @browse_node_id.setter
    def browse_node_id(self, browse_node_id):
        """Sets the browse_node_id of this SearchItemsRequest.


        :param browse_node_id: The browse_node_id of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._browse_node_id = browse_node_id

    @property
    def condition(self):
        """Gets the condition of this SearchItemsRequest.  # noqa: E501


        :return: The condition of this SearchItemsRequest.  # noqa: E501
        :rtype: Condition
        """
        return self._condition

    @condition.setter
    def condition(self, condition):
        """Sets the condition of this SearchItemsRequest.


        :param condition: The condition of this SearchItemsRequest.  # noqa: E501
        :type: Condition
        """

        self._condition = condition

    @property
    def currency_of_preference(self):
        """Gets the currency_of_preference of this SearchItemsRequest.  # noqa: E501


        :return: The currency_of_preference of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._currency_of_preference

    @currency_of_preference.setter
    def currency_of_preference(self, currency_of_preference):
        """Sets the currency_of_preference of this SearchItemsRequest.


        :param currency_of_preference: The currency_of_preference of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._currency_of_preference = currency_of_preference

    @property
    def delivery_flags(self):
        """Gets the delivery_flags of this SearchItemsRequest.  # noqa: E501


        :return: The delivery_flags of this SearchItemsRequest.  # noqa: E501
        :rtype: list[DeliveryFlag]
        """
        return self._delivery_flags

    @delivery_flags.setter
    def delivery_flags(self, delivery_flags):
        """Sets the delivery_flags of this SearchItemsRequest.


        :param delivery_flags: The delivery_flags of this SearchItemsRequest.  # noqa: E501
        :type: list[DeliveryFlag]
        """

        self._delivery_flags = delivery_flags

    @property
    def item_count(self):
        """Gets the item_count of this SearchItemsRequest.  # noqa: E501


        :return: The item_count of this SearchItemsRequest.  # noqa: E501
        :rtype: int
        """
        return self._item_count

    @item_count.setter
    def item_count(self, item_count):
        """Sets the item_count of this SearchItemsRequest.


        :param item_count: The item_count of this SearchItemsRequest.  # noqa: E501
        :type: int
        """

        self._item_count = item_count

    @property
    def item_page(self):
        """Gets the item_page of this SearchItemsRequest.  # noqa: E501


        :return: The item_page of this SearchItemsRequest.  # noqa: E501
        :rtype: int
        """
        return self._item_page

    @item_page.setter
    def item_page(self, item_page):
        """Sets the item_page of this SearchItemsRequest.


        :param item_page: The item_page of this SearchItemsRequest.  # noqa: E501
        :type: int
        """

        self._item_page = item_page

    @property
    def keywords(self):
        """Gets the keywords of this SearchItemsRequest.  # noqa: E501


        :return: The keywords of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._keywords

    @keywords.setter
    def keywords(self, keywords):
        """Sets the keywords of this SearchItemsRequest.


        :param keywords: The keywords of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._keywords = keywords

    @property
    def languages_of_preference(self):
        """Gets the languages_of_preference of this SearchItemsRequest.  # noqa: E501


        :return: The languages_of_preference of this SearchItemsRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._languages_of_preference

    @languages_of_preference.setter
    def languages_of_preference(self, languages_of_preference):
        """Sets the languages_of_preference of this SearchItemsRequest.


        :param languages_of_preference: The languages_of_preference of this SearchItemsRequest.  # noqa: E501
        :type: list[str]
        """

        self._languages_of_preference = languages_of_preference

    @property
    def marketplace(self):
        """Gets the marketplace of this SearchItemsRequest.  # noqa: E501


        :return: The marketplace of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._marketplace

    @marketplace.setter
    def marketplace(self, marketplace):
        """Sets the marketplace of this SearchItemsRequest.


        :param marketplace: The marketplace of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._marketplace = marketplace

    @property
    def max_price(self):
        """Gets the max_price of this SearchItemsRequest.  # noqa: E501


        :return: The max_price of this SearchItemsRequest.  # noqa: E501
        :rtype: MaxPrice
        """
        return self._max_price

    @max_price.setter
    def max_price(self, max_price):
        """Sets the max_price of this SearchItemsRequest.


        :param max_price: The max_price of this SearchItemsRequest.  # noqa: E501
        :type: MaxPrice
        """

        self._max_price = max_price

    @property
    def merchant(self):
        """Gets the merchant of this SearchItemsRequest.  # noqa: E501


        :return: The merchant of this SearchItemsRequest.  # noqa: E501
        :rtype: Merchant
        """
        return self._merchant

    @merchant.setter
    def merchant(self, merchant):
        """Sets the merchant of this SearchItemsRequest.


        :param merchant: The merchant of this SearchItemsRequest.  # noqa: E501
        :type: Merchant
        """

        self._merchant = merchant

    @property
    def min_price(self):
        """Gets the min_price of this SearchItemsRequest.  # noqa: E501


        :return: The min_price of this SearchItemsRequest.  # noqa: E501
        :rtype: MinPrice
        """
        return self._min_price

    @min_price.setter
    def min_price(self, min_price):
        """Sets the min_price of this SearchItemsRequest.


        :param min_price: The min_price of this SearchItemsRequest.  # noqa: E501
        :type: MinPrice
        """

        self._min_price = min_price

    @property
    def min_reviews_rating(self):
        """Gets the min_reviews_rating of this SearchItemsRequest.  # noqa: E501


        :return: The min_reviews_rating of this SearchItemsRequest.  # noqa: E501
        :rtype: MinReviewsRating
        """
        return self._min_reviews_rating

    @min_reviews_rating.setter
    def min_reviews_rating(self, min_reviews_rating):
        """Sets the min_reviews_rating of this SearchItemsRequest.


        :param min_reviews_rating: The min_reviews_rating of this SearchItemsRequest.  # noqa: E501
        :type: MinReviewsRating
        """

        self._min_reviews_rating = min_reviews_rating

    @property
    def min_saving_percent(self):
        """Gets the min_saving_percent of this SearchItemsRequest.  # noqa: E501


        :return: The min_saving_percent of this SearchItemsRequest.  # noqa: E501
        :rtype: MinSavingPercent
        """
        return self._min_saving_percent

    @min_saving_percent.setter
    def min_saving_percent(self, min_saving_percent):
        """Sets the min_saving_percent of this SearchItemsRequest.


        :param min_saving_percent: The min_saving_percent of this SearchItemsRequest.  # noqa: E501
        :type: MinSavingPercent
        """

        self._min_saving_percent = min_saving_percent

    @property
    def offer_count(self):
        """Gets the offer_count of this SearchItemsRequest.  # noqa: E501


        :return: The offer_count of this SearchItemsRequest.  # noqa: E501
        :rtype: OfferCount
        """
        return self._offer_count

    @offer_count.setter
    def offer_count(self, offer_count):
        """Sets the offer_count of this SearchItemsRequest.


        :param offer_count: The offer_count of this SearchItemsRequest.  # noqa: E501
        :type: OfferCount
        """

        self._offer_count = offer_count

    @property
    def partner_tag(self):
        """Gets the partner_tag of this SearchItemsRequest.  # noqa: E501


        :return: The partner_tag of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._partner_tag

    @partner_tag.setter
    def partner_tag(self, partner_tag):
        """Sets the partner_tag of this SearchItemsRequest.


        :param partner_tag: The partner_tag of this SearchItemsRequest.  # noqa: E501
        :type: str
        """
        if partner_tag is None:
            raise ValueError("Invalid value for `partner_tag`, must not be `None`")  # noqa: E501

        self._partner_tag = partner_tag

    @property
    def partner_type(self):
        """Gets the partner_type of this SearchItemsRequest.  # noqa: E501


        :return: The partner_type of this SearchItemsRequest.  # noqa: E501
        :rtype: PartnerType
        """
        return self._partner_type

    @partner_type.setter
    def partner_type(self, partner_type):
        """Sets the partner_type of this SearchItemsRequest.


        :param partner_type: The partner_type of this SearchItemsRequest.  # noqa: E501
        :type: PartnerType
        """
        if partner_type is None:
            raise ValueError("Invalid value for `partner_type`, must not be `None`")  # noqa: E501

        self._partner_type = partner_type

    @property
    def properties(self):
        """Gets the properties of this SearchItemsRequest.  # noqa: E501


        :return: The properties of this SearchItemsRequest.  # noqa: E501
        :rtype: Properties
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this SearchItemsRequest.


        :param properties: The properties of this SearchItemsRequest.  # noqa: E501
        :type: Properties
        """

        self._properties = properties

    @property
    def resources(self):
        """Gets the resources of this SearchItemsRequest.  # noqa: E501


        :return: The resources of this SearchItemsRequest.  # noqa: E501
        :rtype: list[SearchItemsResource]
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this SearchItemsRequest.


        :param resources: The resources of this SearchItemsRequest.  # noqa: E501
        :type: list[SearchItemsResource]
        """

        self._resources = resources

    @property
    def search_index(self):
        """Gets the search_index of this SearchItemsRequest.  # noqa: E501


        :return: The search_index of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._search_index

    @search_index.setter
    def search_index(self, search_index):
        """Sets the search_index of this SearchItemsRequest.


        :param search_index: The search_index of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._search_index = search_index

    @property
    def sort_by(self):
        """Gets the sort_by of this SearchItemsRequest.  # noqa: E501


        :return: The sort_by of this SearchItemsRequest.  # noqa: E501
        :rtype: SortBy
        """
        return self._sort_by

    @sort_by.setter
    def sort_by(self, sort_by):
        """Sets the sort_by of this SearchItemsRequest.


        :param sort_by: The sort_by of this SearchItemsRequest.  # noqa: E501
        :type: SortBy
        """

        self._sort_by = sort_by

    @property
    def title(self):
        """Gets the title of this SearchItemsRequest.  # noqa: E501


        :return: The title of this SearchItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this SearchItemsRequest.


        :param title: The title of this SearchItemsRequest.  # noqa: E501
        :type: str
        """

        self._title = title

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(SearchItemsRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SearchItemsRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.dimension_based_attribute import DimensionBasedAttribute  # noqa: F401,E501
from paapi5_python_sdk.models.single_boolean_valued_attribute import (
    SingleBooleanValuedAttribute,
)  # noqa: F401,E501
from paapi5_python_sdk.models.single_integer_valued_attribute import (
    SingleIntegerValuedAttribute,
)  # noqa: F401,E501
from paapi5_python_sdk.models.single_string_valued_attribute import (
    SingleStringValuedAttribute,
)  # noqa: F401,E501


class ProductInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "color": "SingleStringValuedAttribute",
        "is_adult_product": "SingleBooleanValuedAttribute",
        "item_dimensions": "DimensionBasedAttribute",
        "release_date": "SingleStringValuedAttribute",
        "size": "SingleStringValuedAttribute",
        "unit_count": "SingleIntegerValuedAttribute",
    }

    attribute_map = {
        "color": "Color",
        "is_adult_product": "IsAdultProduct",
        "item_dimensions": "ItemDimensions",
        "release_date": "ReleaseDate",
        "size": "Size",
        "unit_count": "UnitCount",
    }

    def __init__(
        self,
        color=None,
        is_adult_product=None,
        item_dimensions=None,
        release_date=None,
        size=None,
        unit_count=None,
    ):  # noqa: E501
        """ProductInfo - a model defined in Swagger"""  # noqa: E501

        self._color = None
        self._is_adult_product = None
        self._item_dimensions = None
        self._release_date = None
        self._size = None
        self._unit_count = None
        self.discriminator = None

        if color is not None:
            self.color = color
        if is_adult_product is not None:
            self.is_adult_product = is_adult_product
        if item_dimensions is not None:
            self.item_dimensions = item_dimensions
        if release_date is not None:
            self.release_date = release_date
        if size is not None:
            self.size = size
        if unit_count is not None:
            self.unit_count = unit_count

    @property
    def color(self):
        """Gets the color of this ProductInfo.  # noqa: E501


        :return: The color of this ProductInfo.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._color

    @color.setter
    def color(self, color):
        """Sets the color of this ProductInfo.


        :param color: The color of this ProductInfo.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._color = color

    @property
    def is_adult_product(self):
        """Gets the is_adult_product of this ProductInfo.  # noqa: E501


        :return: The is_adult_product of this ProductInfo.  # noqa: E501
        :rtype: SingleBooleanValuedAttribute
        """
        return self._is_adult_product

    @is_adult_product.setter
    def is_adult_product(self, is_adult_product):
        """Sets the is_adult_product of this ProductInfo.


        :param is_adult_product: The is_adult_product of this ProductInfo.  # noqa: E501
        :type: SingleBooleanValuedAttribute
        """

        self._is_adult_product = is_adult_product

    @property
    def item_dimensions(self):
        """Gets the item_dimensions of this ProductInfo.  # noqa: E501


        :return: The item_dimensions of this ProductInfo.  # noqa: E501
        :rtype: DimensionBasedAttribute
        """
        return self._item_dimensions

    @item_dimensions.setter
    def item_dimensions(self, item_dimensions):
        """Sets the item_dimensions of this ProductInfo.


        :param item_dimensions: The item_dimensions of this ProductInfo.  # noqa: E501
        :type: DimensionBasedAttribute
        """

        self._item_dimensions = item_dimensions

    @property
    def release_date(self):
        """Gets the release_date of this ProductInfo.  # noqa: E501


        :return: The release_date of this ProductInfo.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._release_date

    @release_date.setter
    def release_date(self, release_date):
        """Sets the release_date of this ProductInfo.


        :param release_date: The release_date of this ProductInfo.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._release_date = release_date

    @property
    def size(self):
        """Gets the size of this ProductInfo.  # noqa: E501


        :return: The size of this ProductInfo.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this ProductInfo.


        :param size: The size of this ProductInfo.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._size = size

    @property
    def unit_count(self):
        """Gets the unit_count of this ProductInfo.  # noqa: E501


        :return: The unit_count of this ProductInfo.  # noqa: E501
        :rtype: SingleIntegerValuedAttribute
        """
        return self._unit_count

    @unit_count.setter
    def unit_count(self, unit_count):
        """Sets the unit_count of this ProductInfo.


        :param unit_count: The unit_count of this ProductInfo.  # noqa: E501
        :type: SingleIntegerValuedAttribute
        """

        self._unit_count = unit_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(ProductInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProductInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Initialize database schema and run migrations."""

import argparse
import logging
import sys

from opportunity_core.models.database import Base
from opportunity_core.services.database_manager import DatabaseManager
from opportunity_core.utils.load_config import DatabaseConfig

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
)
logger = logging.getLogger(__name__)


def init_database(database_url: str, drop_existing: bool = False):
    """
    Initialize database schema.

    Args:
        database_url: SQLAlchemy database URL
        drop_existing: If True, drop all existing tables first (DESTRUCTIVE!)
    """
    logger.info("=" * 80)
    logger.info("DATABASE INITIALIZATION")
    logger.info("=" * 80)

    # Initialize database manager
    db_manager = DatabaseManager(database_url, echo=True)

    # Health check
    if not db_manager.health_check():
        logger.error("❌ Database connection failed! Check your DATABASE_URL.")
        sys.exit(1)

    logger.info("✅ Database connection successful")

    # Drop tables if requested
    if drop_existing:
        logger.warning("⚠️  DROPPING ALL EXISTING TABLES...")
        response = input("Are you sure? This will DELETE ALL DATA! Type 'yes' to confirm: ")
        if response.lower() != "yes":
            logger.info("Aborted.")
            return

        db_manager.drop_tables()
        logger.info("✅ All tables dropped")

    # Create tables
    logger.info("Creating database schema...")
    db_manager.create_tables()
    logger.info("✅ Database schema created successfully")

    # Print table information
    logger.info("\n" + "=" * 80)
    logger.info("CREATED TABLES:")
    logger.info("=" * 80)

    tables = [
        ("products", "Stores discovered ASINs for tracking"),
        ("price_history", "Time series price data"),
        ("price_statistics", "Materialized price stats (min/max/avg)"),
        ("alerts_sent", "Tracks sent alerts to prevent duplicates"),
    ]

    for table_name, description in tables:
        logger.info(f"  ✓ {table_name:20} - {description}")

    logger.info("\n" + "=" * 80)
    logger.info("✅ DATABASE READY!")
    logger.info("=" * 80)

    db_manager.close()


def main():
    """Main entry point."""
    parser = argparse.ArgumentParser(description="Initialize Amazon Deals Tracker database")
    parser.add_argument(
        "--database-url",
        help="Database URL (default: from .env DATABASE_URL)",
        default=None,
    )
    parser.add_argument(
        "--drop",
        action="store_true",
        help="Drop all existing tables before creating (DESTRUCTIVE!)",
    )

    args = parser.parse_args()

    # Get database URL
    if args.database_url:
        database_url = args.database_url
    else:
        config = DatabaseConfig()
        database_url = config.get_database_url()

    logger.info(f"Database URL: {database_url.split('@')[0]}@***")  # Mask password

    try:
        init_database(database_url, drop_existing=args.drop)
    except Exception as e:
        logger.error(f"❌ Initialization failed: {e}")
        sys.exit(1)


if __name__ == "__main__":
    main()



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.get_browse_nodes_resource import GetBrowseNodesResource  # noqa: F401,E501
from paapi5_python_sdk.models.partner_type import PartnerType  # noqa: F401,E501


class GetBrowseNodesRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "browse_node_ids": "list[str]",
        "languages_of_preference": "list[str]",
        "marketplace": "str",
        "partner_tag": "str",
        "partner_type": "PartnerType",
        "resources": "list[GetBrowseNodesResource]",
    }

    attribute_map = {
        "browse_node_ids": "BrowseNodeIds",
        "languages_of_preference": "LanguagesOfPreference",
        "marketplace": "Marketplace",
        "partner_tag": "PartnerTag",
        "partner_type": "PartnerType",
        "resources": "Resources",
    }

    def __init__(
        self,
        browse_node_ids=None,
        languages_of_preference=None,
        marketplace=None,
        partner_tag=None,
        partner_type=None,
        resources=None,
    ):  # noqa: E501
        """GetBrowseNodesRequest - a model defined in Swagger"""  # noqa: E501

        self._browse_node_ids = None
        self._languages_of_preference = None
        self._marketplace = None
        self._partner_tag = None
        self._partner_type = None
        self._resources = None
        self.discriminator = None

        self.browse_node_ids = browse_node_ids
        if languages_of_preference is not None:
            self.languages_of_preference = languages_of_preference
        if marketplace is not None:
            self.marketplace = marketplace
        self.partner_tag = partner_tag
        self.partner_type = partner_type
        if resources is not None:
            self.resources = resources

    @property
    def browse_node_ids(self):
        """Gets the browse_node_ids of this GetBrowseNodesRequest.  # noqa: E501


        :return: The browse_node_ids of this GetBrowseNodesRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._browse_node_ids

    @browse_node_ids.setter
    def browse_node_ids(self, browse_node_ids):
        """Sets the browse_node_ids of this GetBrowseNodesRequest.


        :param browse_node_ids: The browse_node_ids of this GetBrowseNodesRequest.  # noqa: E501
        :type: list[str]
        """
        if browse_node_ids is None:
            raise ValueError("Invalid value for `browse_node_ids`, must not be `None`")  # noqa: E501

        self._browse_node_ids = browse_node_ids

    @property
    def languages_of_preference(self):
        """Gets the languages_of_preference of this GetBrowseNodesRequest.  # noqa: E501


        :return: The languages_of_preference of this GetBrowseNodesRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._languages_of_preference

    @languages_of_preference.setter
    def languages_of_preference(self, languages_of_preference):
        """Sets the languages_of_preference of this GetBrowseNodesRequest.


        :param languages_of_preference: The languages_of_preference of this GetBrowseNodesRequest.  # noqa: E501
        :type: list[str]
        """

        self._languages_of_preference = languages_of_preference

    @property
    def marketplace(self):
        """Gets the marketplace of this GetBrowseNodesRequest.  # noqa: E501


        :return: The marketplace of this GetBrowseNodesRequest.  # noqa: E501
        :rtype: str
        """
        return self._marketplace

    @marketplace.setter
    def marketplace(self, marketplace):
        """Sets the marketplace of this GetBrowseNodesRequest.


        :param marketplace: The marketplace of this GetBrowseNodesRequest.  # noqa: E501
        :type: str
        """

        self._marketplace = marketplace

    @property
    def partner_tag(self):
        """Gets the partner_tag of this GetBrowseNodesRequest.  # noqa: E501


        :return: The partner_tag of this GetBrowseNodesRequest.  # noqa: E501
        :rtype: str
        """
        return self._partner_tag

    @partner_tag.setter
    def partner_tag(self, partner_tag):
        """Sets the partner_tag of this GetBrowseNodesRequest.


        :param partner_tag: The partner_tag of this GetBrowseNodesRequest.  # noqa: E501
        :type: str
        """
        if partner_tag is None:
            raise ValueError("Invalid value for `partner_tag`, must not be `None`")  # noqa: E501

        self._partner_tag = partner_tag

    @property
    def partner_type(self):
        """Gets the partner_type of this GetBrowseNodesRequest.  # noqa: E501


        :return: The partner_type of this GetBrowseNodesRequest.  # noqa: E501
        :rtype: PartnerType
        """
        return self._partner_type

    @partner_type.setter
    def partner_type(self, partner_type):
        """Sets the partner_type of this GetBrowseNodesRequest.


        :param partner_type: The partner_type of this GetBrowseNodesRequest.  # noqa: E501
        :type: PartnerType
        """
        if partner_type is None:
            raise ValueError("Invalid value for `partner_type`, must not be `None`")  # noqa: E501

        self._partner_type = partner_type

    @property
    def resources(self):
        """Gets the resources of this GetBrowseNodesRequest.  # noqa: E501


        :return: The resources of this GetBrowseNodesRequest.  # noqa: E501
        :rtype: list[GetBrowseNodesResource]
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this GetBrowseNodesRequest.


        :param resources: The resources of this GetBrowseNodesRequest.  # noqa: E501
        :type: list[GetBrowseNodesResource]
        """

        self._resources = resources

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(GetBrowseNodesRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetBrowseNodesRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.browse_node_info import BrowseNodeInfo  # noqa: F401,E501
from paapi5_python_sdk.models.customer_reviews import CustomerReviews  # noqa: F401,E501
from paapi5_python_sdk.models.images import Images  # noqa: F401,E501
from paapi5_python_sdk.models.item_info import ItemInfo  # noqa: F401,E501
from paapi5_python_sdk.models.offers import Offers  # noqa: F401,E501
from paapi5_python_sdk.models.offers_v2 import OffersV2  # noqa: F401,E501
from paapi5_python_sdk.models.rental_offers import RentalOffers  # noqa: F401,E501
from paapi5_python_sdk.models.variation_attribute import VariationAttribute  # noqa: F401,E501


class Item(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "asin": "str",
        "browse_node_info": "BrowseNodeInfo",
        "customer_reviews": "CustomerReviews",
        "detail_page_url": "str",
        "images": "Images",
        "item_info": "ItemInfo",
        "offers": "Offers",
        "offers_v2": "OffersV2",
        "parent_asin": "str",
        "rental_offers": "RentalOffers",
        "score": "float",
        "variation_attributes": "list[VariationAttribute]",
    }

    attribute_map = {
        "asin": "ASIN",
        "browse_node_info": "BrowseNodeInfo",
        "customer_reviews": "CustomerReviews",
        "detail_page_url": "DetailPageURL",
        "images": "Images",
        "item_info": "ItemInfo",
        "offers": "Offers",
        "offers_v2": "OffersV2",
        "parent_asin": "ParentASIN",
        "rental_offers": "RentalOffers",
        "score": "Score",
        "variation_attributes": "VariationAttributes",
    }

    def __init__(
        self,
        asin=None,
        browse_node_info=None,
        customer_reviews=None,
        detail_page_url=None,
        images=None,
        item_info=None,
        offers=None,
        offers_v2=None,
        parent_asin=None,
        rental_offers=None,
        score=None,
        variation_attributes=None,
    ):  # noqa: E501
        """Item - a model defined in Swagger"""  # noqa: E501

        self._asin = None
        self._browse_node_info = None
        self._customer_reviews = None
        self._detail_page_url = None
        self._images = None
        self._item_info = None
        self._offers = None
        self._offers_v2 = None
        self._parent_asin = None
        self._rental_offers = None
        self._score = None
        self._variation_attributes = None
        self.discriminator = None

        if asin is not None:
            self.asin = asin
        if browse_node_info is not None:
            self.browse_node_info = browse_node_info
        if customer_reviews is not None:
            self.customer_reviews = customer_reviews
        if detail_page_url is not None:
            self.detail_page_url = detail_page_url
        if images is not None:
            self.images = images
        if item_info is not None:
            self.item_info = item_info
        if offers is not None:
            self.offers = offers
        if offers_v2 is not None:
            self.offers_v2 = offers_v2
        if parent_asin is not None:
            self.parent_asin = parent_asin
        if rental_offers is not None:
            self.rental_offers = rental_offers
        if score is not None:
            self.score = score
        if variation_attributes is not None:
            self.variation_attributes = variation_attributes

    @property
    def asin(self):
        """Gets the asin of this Item.  # noqa: E501


        :return: The asin of this Item.  # noqa: E501
        :rtype: str
        """
        return self._asin

    @asin.setter
    def asin(self, asin):
        """Sets the asin of this Item.


        :param asin: The asin of this Item.  # noqa: E501
        :type: str
        """

        self._asin = asin

    @property
    def browse_node_info(self):
        """Gets the browse_node_info of this Item.  # noqa: E501


        :return: The browse_node_info of this Item.  # noqa: E501
        :rtype: BrowseNodeInfo
        """
        return self._browse_node_info

    @browse_node_info.setter
    def browse_node_info(self, browse_node_info):
        """Sets the browse_node_info of this Item.


        :param browse_node_info: The browse_node_info of this Item.  # noqa: E501
        :type: BrowseNodeInfo
        """

        self._browse_node_info = browse_node_info

    @property
    def customer_reviews(self):
        """Gets the customer_reviews of this Item.  # noqa: E501


        :return: The customer_reviews of this Item.  # noqa: E501
        :rtype: CustomerReviews
        """
        return self._customer_reviews

    @customer_reviews.setter
    def customer_reviews(self, customer_reviews):
        """Sets the customer_reviews of this Item.


        :param customer_reviews: The customer_reviews of this Item.  # noqa: E501
        :type: CustomerReviews
        """

        self._customer_reviews = customer_reviews

    @property
    def detail_page_url(self):
        """Gets the detail_page_url of this Item.  # noqa: E501


        :return: The detail_page_url of this Item.  # noqa: E501
        :rtype: str
        """
        return self._detail_page_url

    @detail_page_url.setter
    def detail_page_url(self, detail_page_url):
        """Sets the detail_page_url of this Item.


        :param detail_page_url: The detail_page_url of this Item.  # noqa: E501
        :type: str
        """

        self._detail_page_url = detail_page_url

    @property
    def images(self):
        """Gets the images of this Item.  # noqa: E501


        :return: The images of this Item.  # noqa: E501
        :rtype: Images
        """
        return self._images

    @images.setter
    def images(self, images):
        """Sets the images of this Item.


        :param images: The images of this Item.  # noqa: E501
        :type: Images
        """

        self._images = images

    @property
    def item_info(self):
        """Gets the item_info of this Item.  # noqa: E501


        :return: The item_info of this Item.  # noqa: E501
        :rtype: ItemInfo
        """
        return self._item_info

    @item_info.setter
    def item_info(self, item_info):
        """Sets the item_info of this Item.


        :param item_info: The item_info of this Item.  # noqa: E501
        :type: ItemInfo
        """

        self._item_info = item_info

    @property
    def offers(self):
        """Gets the offers of this Item.  # noqa: E501


        :return: The offers of this Item.  # noqa: E501
        :rtype: Offers
        """
        return self._offers

    @offers.setter
    def offers(self, offers):
        """Sets the offers of this Item.


        :param offers: The offers of this Item.  # noqa: E501
        :type: Offers
        """

        self._offers = offers

    @property
    def offers_v2(self):
        """Gets the offers_v2 of this Item.  # noqa: E501


        :return: The offers_v2 of this Item.  # noqa: E501
        :rtype: OffersV2
        """
        return self._offers_v2

    @offers_v2.setter
    def offers_v2(self, offers_v2):
        """Sets the offers_v2 of this Item.


        :param offers_v2: The offers_v2 of this Item.  # noqa: E501
        :type: OffersV2
        """

        self._offers_v2 = offers_v2

    @property
    def parent_asin(self):
        """Gets the parent_asin of this Item.  # noqa: E501


        :return: The parent_asin of this Item.  # noqa: E501
        :rtype: str
        """
        return self._parent_asin

    @parent_asin.setter
    def parent_asin(self, parent_asin):
        """Sets the parent_asin of this Item.


        :param parent_asin: The parent_asin of this Item.  # noqa: E501
        :type: str
        """

        self._parent_asin = parent_asin

    @property
    def rental_offers(self):
        """Gets the rental_offers of this Item.  # noqa: E501


        :return: The rental_offers of this Item.  # noqa: E501
        :rtype: RentalOffers
        """
        return self._rental_offers

    @rental_offers.setter
    def rental_offers(self, rental_offers):
        """Sets the rental_offers of this Item.


        :param rental_offers: The rental_offers of this Item.  # noqa: E501
        :type: RentalOffers
        """

        self._rental_offers = rental_offers

    @property
    def score(self):
        """Gets the score of this Item.  # noqa: E501


        :return: The score of this Item.  # noqa: E501
        :rtype: float
        """
        return self._score

    @score.setter
    def score(self, score):
        """Sets the score of this Item.


        :param score: The score of this Item.  # noqa: E501
        :type: float
        """

        self._score = score

    @property
    def variation_attributes(self):
        """Gets the variation_attributes of this Item.  # noqa: E501


        :return: The variation_attributes of this Item.  # noqa: E501
        :rtype: list[VariationAttribute]
        """
        return self._variation_attributes

    @variation_attributes.setter
    def variation_attributes(self, variation_attributes):
        """Sets the variation_attributes of this Item.


        :param variation_attributes: The variation_attributes of this Item.  # noqa: E501
        :type: list[VariationAttribute]
        """

        self._variation_attributes = variation_attributes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(Item, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Item):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.money import Money  # noqa: F401,E501
from paapi5_python_sdk.models.offer_saving_basis import OfferSavingBasis  # noqa: F401,E501
from paapi5_python_sdk.models.offer_savings_v2 import OfferSavingsV2  # noqa: F401,E501


class OfferPriceV2(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "money": "Money",
        "price_per_unit": "Money",
        "savings": "OfferSavingsV2",
        "saving_basis": "OfferSavingBasis",
    }

    attribute_map = {
        "money": "Money",
        "price_per_unit": "PricePerUnit",
        "savings": "Savings",
        "saving_basis": "SavingBasis",
    }

    def __init__(self, money=None, price_per_unit=None, savings=None, saving_basis=None):  # noqa: E501
        """OfferPriceV2 - a model defined in Swagger"""  # noqa: E501

        self._money = None
        self._price_per_unit = None
        self._savings = None
        self._saving_basis = None
        self.discriminator = None

        if money is not None:
            self.money = money
        if price_per_unit is not None:
            self.price_per_unit = price_per_unit
        if savings is not None:
            self.savings = savings
        if saving_basis is not None:
            self.saving_basis = saving_basis

    @property
    def money(self):
        """Gets the money of this OfferPriceV2.  # noqa: E501


        :return: The money of this OfferPriceV2.  # noqa: E501
        :rtype: Money
        """
        return self._money

    @money.setter
    def money(self, money):
        """Sets the money of this OfferPriceV2.


        :param money: The money of this OfferPriceV2.  # noqa: E501
        :type: Money
        """

        self._money = money

    @property
    def price_per_unit(self):
        """Gets the price_per_unit of this OfferPriceV2.  # noqa: E501


        :return: The price_per_unit of this OfferPriceV2.  # noqa: E501
        :rtype: Money
        """
        return self._price_per_unit

    @price_per_unit.setter
    def price_per_unit(self, price_per_unit):
        """Sets the price_per_unit of this OfferPriceV2.


        :param price_per_unit: The price_per_unit of this OfferPriceV2.  # noqa: E501
        :type: Money
        """

        self._price_per_unit = price_per_unit

    @property
    def savings(self):
        """Gets the savings of this OfferPriceV2.  # noqa: E501


        :return: The savings of this OfferPriceV2.  # noqa: E501
        :rtype: OfferSavingsV2
        """
        return self._savings

    @savings.setter
    def savings(self, savings):
        """Sets the savings of this OfferPriceV2.


        :param savings: The savings of this OfferPriceV2.  # noqa: E501
        :type: OfferSavingsV2
        """

        self._savings = savings

    @property
    def saving_basis(self):
        """Gets the saving_basis of this OfferPriceV2.  # noqa: E501


        :return: The saving_basis of this OfferPriceV2.  # noqa: E501
        :rtype: OfferSavingBasis
        """
        return self._saving_basis

    @saving_basis.setter
    def saving_basis(self, saving_basis):
        """Sets the saving_basis of this OfferPriceV2.


        :param saving_basis: The saving_basis of this OfferPriceV2.  # noqa: E501
        :type: OfferSavingBasis
        """

        self._saving_basis = saving_basis

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferPriceV2, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferPriceV2):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six


class OfferMerchantInfoV2(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {"name": "str", "id": "str"}

    attribute_map = {"name": "Name", "id": "Id"}

    def __init__(self, name=None, id=None):  # noqa: E501
        """OfferMerchantInfoV2 - a model defined in Swagger"""  # noqa: E501

        self._name = None
        self._id = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if id is not None:
            self.id = id

    @property
    def name(self):
        """Gets the name of this OfferMerchantInfoV2.  # noqa: E501


        :return: The name of this OfferMerchantInfoV2.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OfferMerchantInfoV2.


        :param name: The name of this OfferMerchantInfoV2.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this OfferMerchantInfoV2.  # noqa: E501


        :return: The id of this OfferMerchantInfoV2.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OfferMerchantInfoV2.


        :param id: The id of this OfferMerchantInfoV2.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferMerchantInfoV2, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferMerchantInfoV2):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.single_string_valued_attribute import (
    SingleStringValuedAttribute,
)  # noqa: F401,E501


class Classifications(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "binding": "SingleStringValuedAttribute",
        "product_group": "SingleStringValuedAttribute",
    }

    attribute_map = {"binding": "Binding", "product_group": "ProductGroup"}

    def __init__(self, binding=None, product_group=None):  # noqa: E501
        """Classifications - a model defined in Swagger"""  # noqa: E501

        self._binding = None
        self._product_group = None
        self.discriminator = None

        if binding is not None:
            self.binding = binding
        if product_group is not None:
            self.product_group = product_group

    @property
    def binding(self):
        """Gets the binding of this Classifications.  # noqa: E501


        :return: The binding of this Classifications.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._binding

    @binding.setter
    def binding(self, binding):
        """Sets the binding of this Classifications.


        :param binding: The binding of this Classifications.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._binding = binding

    @property
    def product_group(self):
        """Gets the product_group of this Classifications.  # noqa: E501


        :return: The product_group of this Classifications.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._product_group

    @product_group.setter
    def product_group(self, product_group):
        """Sets the product_group of this Classifications.


        :param product_group: The product_group of this Classifications.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._product_group = product_group

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(Classifications, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Classifications):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
ProductAdvertisingAPI

https://webservices.amazon.com/paapi5/documentation/index.html

"""

import hashlib
import hmac
import json


class AWSV4Auth:
    def __init__(
        self,
        access_key,
        secret_key,
        host,
        region,
        service,
        method_name,
        timestamp,
        headers={},
        path="",
        payload="",
    ):
        self.access_key = access_key
        self.secret_key = secret_key
        self.host = host
        self.region = region
        self.service = service
        self.method_name = method_name
        self.headers = headers
        self.timestamp = timestamp
        self.payload = payload
        self.path = path

        # Date and time stamp
        self.xAmzDateTime = self.timestamp.strftime("%Y%m%dT%H%M%SZ")
        self.xAmzDate = self.timestamp.strftime("%Y%m%d")

    def get_headers(self):
        canonical_request = self.prepare_canonical_url()
        string_to_sign = self.prepare_string_to_sign(canonical_request=canonical_request)
        signing_key = self.get_signature_key(self.secret_key, self.xAmzDate, self.region, self.service)
        signature = self.get_signature(signing_key=signing_key, string_to_sign=string_to_sign)

        authorization_header = (
            self.algorithm
            + " "
            + "Credential="
            + self.access_key
            + "/"
            + self.credential_scope
            + ", "
            + "SignedHeaders="
            + self.signed_header
            + ", "
            + "Signature="
            + signature
        )
        self.headers["Authorization"] = authorization_header
        return self.headers

    def prepare_canonical_url(self):
        canonical_uri = self.method_name + "\n" + self.path
        canonical_querystring = ""
        canonical_header = ""
        self.signed_header = ""
        sorted_keys = sorted(self.headers, key=str.lower)
        for key in sorted_keys:
            self.signed_header = self.signed_header + key.lower() + ";"
            canonical_header = canonical_header + key.lower() + ":" + self.headers[key] + "\n"
        self.signed_header = self.signed_header[:-1]
        payload_hash = hashlib.sha256(json.dumps(self.payload).encode("utf-8")).hexdigest()
        canonical_request = (
            canonical_uri
            + "\n"
            + canonical_querystring
            + "\n"
            + canonical_header
            + "\n"
            + self.signed_header
            + "\n"
            + payload_hash
        )
        return canonical_request

    def prepare_string_to_sign(self, canonical_request):
        self.algorithm = "AWS4-HMAC-SHA256"
        self.credential_scope = self.xAmzDate + "/" + self.region + "/" + self.service + "/" + "aws4_request"
        string_to_sign = (
            self.algorithm
            + "\n"
            + self.xAmzDateTime
            + "\n"
            + self.credential_scope
            + "\n"
            + hashlib.sha256(canonical_request.encode("utf-8")).hexdigest()
        )
        return string_to_sign

    def sign(self, key, msg):
        return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

    def get_signature_key(self, key, date_stamp, region_name, service_name):
        k_date = self.sign(("AWS4" + key).encode("utf-8"), date_stamp)
        k_region = self.sign(k_date, region_name)
        k_service = self.sign(k_region, service_name)
        k_signing = self.sign(k_service, "aws4_request")
        return k_signing

    def get_signature(self, signing_key, string_to_sign):
        signature = hmac.new(signing_key, string_to_sign.encode("utf-8"), hashlib.sha256).hexdigest()
        return signature



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six


class DealDetails(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "access_type": "str",
        "badge": "str",
        "early_access_duration_in_milliseconds": "int",
        "end_time": "str",
        "percent_claimed": "int",
        "start_time": "str",
    }

    attribute_map = {
        "access_type": "AccessType",
        "badge": "Badge",
        "early_access_duration_in_milliseconds": "EarlyAccessDurationInMilliseconds",
        "end_time": "EndTime",
        "percent_claimed": "PercentClaimed",
        "start_time": "StartTime",
    }

    def __init__(
        self,
        access_type=None,
        badge=None,
        early_access_duration_in_milliseconds=None,
        end_time=None,
        percent_claimed=None,
        start_time=None,
    ):  # noqa: E501
        """DealDetails - a model defined in Swagger"""  # noqa: E501

        self._access_type = None
        self._badge = None
        self._early_access_duration_in_milliseconds = None
        self._end_time = None
        self._percent_claimed = None
        self._start_time = None
        self.discriminator = None

        if access_type is not None:
            self.access_type = access_type
        if badge is not None:
            self.badge = badge
        if early_access_duration_in_milliseconds is not None:
            self.early_access_duration_in_milliseconds = early_access_duration_in_milliseconds
        if end_time is not None:
            self.end_time = end_time
        if percent_claimed is not None:
            self.percent_claimed = percent_claimed
        if start_time is not None:
            self.start_time = start_time

    @property
    def access_type(self):
        """Gets the access_type of this DealDetails.  # noqa: E501


        :return: The access_type of this DealDetails.  # noqa: E501
        :rtype: str
        """
        return self._access_type

    @access_type.setter
    def access_type(self, access_type):
        """Sets the access_type of this DealDetails.


        :param access_type: The access_type of this DealDetails.  # noqa: E501
        :type: str
        """

        self._access_type = access_type

    @property
    def badge(self):
        """Gets the badge of this DealDetails.  # noqa: E501


        :return: The badge of this DealDetails.  # noqa: E501
        :rtype: str
        """
        return self._badge

    @badge.setter
    def badge(self, badge):
        """Sets the badge of this DealDetails.


        :param badge: The badge of this DealDetails.  # noqa: E501
        :type: str
        """

        self._badge = badge

    @property
    def early_access_duration_in_milliseconds(self):
        """Gets the early_access_duration_in_milliseconds of this DealDetails.  # noqa: E501


        :return: The early_access_duration_in_milliseconds of this DealDetails.  # noqa: E501
        :rtype: int
        """
        return self._early_access_duration_in_milliseconds

    @early_access_duration_in_milliseconds.setter
    def early_access_duration_in_milliseconds(self, early_access_duration_in_milliseconds):
        """Sets the early_access_duration_in_milliseconds of this DealDetails.


        :param early_access_duration_in_milliseconds: The early_access_duration_in_milliseconds of this DealDetails.  # noqa: E501
        :type: int
        """

        self._early_access_duration_in_milliseconds = early_access_duration_in_milliseconds

    @property
    def end_time(self):
        """Gets the end_time of this DealDetails.  # noqa: E501


        :return: The end_time of this DealDetails.  # noqa: E501
        :rtype: str
        """
        return self._end_time

    @end_time.setter
    def end_time(self, end_time):
        """Sets the end_time of this DealDetails.


        :param end_time: The end_time of this DealDetails.  # noqa: E501
        :type: str
        """

        self._end_time = end_time

    @property
    def percent_claimed(self):
        """Gets the percent_claimed of this DealDetails.  # noqa: E501


        :return: The percent_claimed of this DealDetails.  # noqa: E501
        :rtype: int
        """
        return self._percent_claimed

    @percent_claimed.setter
    def percent_claimed(self, percent_claimed):
        """Sets the percent_claimed of this DealDetails.


        :param percent_claimed: The percent_claimed of this DealDetails.  # noqa: E501
        :type: int
        """

        self._percent_claimed = percent_claimed

    @property
    def start_time(self):
        """Gets the start_time of this DealDetails.  # noqa: E501


        :return: The start_time of this DealDetails.  # noqa: E501
        :rtype: str
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this DealDetails.


        :param start_time: The start_time of this DealDetails.  # noqa: E501
        :type: str
        """

        self._start_time = start_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(DealDetails, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DealDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six


class OfferAvailability(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "max_order_quantity": "int",
        "message": "str",
        "min_order_quantity": "int",
        "type": "str",
    }

    attribute_map = {
        "max_order_quantity": "MaxOrderQuantity",
        "message": "Message",
        "min_order_quantity": "MinOrderQuantity",
        "type": "Type",
    }

    def __init__(self, max_order_quantity=None, message=None, min_order_quantity=None, type=None):  # noqa: E501
        """OfferAvailability - a model defined in Swagger"""  # noqa: E501

        self._max_order_quantity = None
        self._message = None
        self._min_order_quantity = None
        self._type = None
        self.discriminator = None

        if max_order_quantity is not None:
            self.max_order_quantity = max_order_quantity
        if message is not None:
            self.message = message
        if min_order_quantity is not None:
            self.min_order_quantity = min_order_quantity
        if type is not None:
            self.type = type

    @property
    def max_order_quantity(self):
        """Gets the max_order_quantity of this OfferAvailability.  # noqa: E501


        :return: The max_order_quantity of this OfferAvailability.  # noqa: E501
        :rtype: int
        """
        return self._max_order_quantity

    @max_order_quantity.setter
    def max_order_quantity(self, max_order_quantity):
        """Sets the max_order_quantity of this OfferAvailability.


        :param max_order_quantity: The max_order_quantity of this OfferAvailability.  # noqa: E501
        :type: int
        """

        self._max_order_quantity = max_order_quantity

    @property
    def message(self):
        """Gets the message of this OfferAvailability.  # noqa: E501


        :return: The message of this OfferAvailability.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this OfferAvailability.


        :param message: The message of this OfferAvailability.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def min_order_quantity(self):
        """Gets the min_order_quantity of this OfferAvailability.  # noqa: E501


        :return: The min_order_quantity of this OfferAvailability.  # noqa: E501
        :rtype: int
        """
        return self._min_order_quantity

    @min_order_quantity.setter
    def min_order_quantity(self, min_order_quantity):
        """Sets the min_order_quantity of this OfferAvailability.


        :param min_order_quantity: The min_order_quantity of this OfferAvailability.  # noqa: E501
        :type: int
        """

        self._min_order_quantity = min_order_quantity

    @property
    def type(self):
        """Gets the type of this OfferAvailability.  # noqa: E501


        :return: The type of this OfferAvailability.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OfferAvailability.


        :param type: The type of this OfferAvailability.  # noqa: E501
        :type: str
        """

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferAvailability, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferAvailability):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Twitter notification service for Amazon deals."""

import logging
from dataclasses import dataclass
from datetime import datetime, timezone
from io import BytesIO

import requests
import tweepy

from opportunity_core.utils.comment_generator import CommentGenerator

logger = logging.getLogger(__name__)


@dataclass
class DealInfo:
    """Data class for deal information."""

    title: str
    current_price: float
    original_price: float
    discount_percentage: int
    currency: str
    asin: str
    detail_page_url: str
    image_url: str | None = None
    category: str | None = None  # Product category
    rating: float | None = None  # Product rating
    review_count: int | None = None  # Number of reviews
    seller_name: str | None = None  # Merchant/seller name
    seller_feedback_count: int | None = None  # Seller feedback count
    seller_feedback_rating: float | None = None  # Seller feedback rating
    is_amazon_fulfilled: bool = False  # Amazon satıcı mı?
    availability_message: str | None = None  # Availability message
    stock_quantity: int | None = None  # Available stock
    is_prime_eligible: bool = False  # Prime üyelerine özel
    is_free_shipping: bool = False  # Ücretsiz kargo
    lowest_price_30d: float | None = None
    lowest_price_90d: float | None = None
    lowest_price_180d: float | None = None
    first_discovered_at: datetime | None = None


class TwitterNotifier:
    """Handles posting deal notifications to Twitter."""

    def __init__(
        self, api_key: str, api_secret: str, access_token: str, access_token_secret: str, bearer_token: str = ""
    ):
        """
        Initialize Twitter API client.

        Args:
            api_key: Twitter API key (Consumer Key)
            api_secret: Twitter API secret (Consumer Secret)
            access_token: Twitter access token
            access_token_secret: Twitter access token secret
            bearer_token: Optional bearer token for v2 API
        """
        try:
            # Use Twitter API v2 Client (supports Free tier)
            self.client = tweepy.Client(
                consumer_key=api_key,
                consumer_secret=api_secret,
                access_token=access_token,
                access_token_secret=access_token_secret,
            )

            # Also keep v1.1 API for media upload
            auth = tweepy.OAuth1UserHandler(api_key, api_secret, access_token, access_token_secret)
            self.api = tweepy.API(auth)

            # Verify credentials
            me = self.client.get_me()
            logger.info(f"Twitter authentication successful. User: @{me.data.username}")
        except Exception as e:
            logger.error(f"Twitter authentication failed: {e}")
            raise

    def shorten_url(self, url: str) -> str:
        """
        Shorten URL using TinyURL service.

        Args:
            url: Long URL to shorten

        Returns:
            Shortened URL or original if service fails
        """
        try:
            response = requests.get(f"http://tinyurl.com/api-create.php?url={url}", timeout=5)
            if response.status_code == 200:
                shortened = response.text
                logger.info(f"URL shortened: {url[:50]}... -> {shortened}")
                return shortened
        except Exception as e:
            logger.warning(f"URL shortening failed: {e}. Using original URL.")
        return url

    def format_deal_message(self, deal: DealInfo) -> str:
        """
        Format deal information into a tweet message with shortened link.

        Args:
            deal: Deal information

        Returns:
            Formatted tweet message
        """
        # Shorten the affiliate link
        short_url = self.shorten_url(deal.detail_page_url)

        # Build compact info for Twitter (280 char limit)
        if deal.is_amazon_fulfilled:
            seller_tag = "Amazon Turkiye"  # Display as "Amazon Turkiye" for Twitter (no Turkish chars)
        elif deal.seller_name:
            # If seller name contains "amazon.com.tr", show as "Amazon Turkiye"
            if "amazon.com.tr" in deal.seller_name.lower():
                seller_tag = "Amazon Turkiye"
            else:
                seller_tag = deal.seller_name[:15]
                # Add seller feedback rating if available (compact format)
                if deal.seller_feedback_rating:
                    seller_tag += f" ⭐{deal.seller_feedback_rating:.1f}"
        else:
            seller_tag = ""
        rating_tag = f"⭐{deal.rating:.1f}" if deal.rating else ""
        review_tag = f"({deal.review_count})" if deal.review_count and deal.review_count < 10000 else ""

        # Add Prime/Shipping badges (compact)
        badges = []
        if deal.is_prime_eligible:
            badges.append("Prime")
        if deal.is_free_shipping:
            badges.append("Ucretsiz Kargo")

        # Build message components (compact for 280 char limit)
        message = f"Fırsat! %{deal.discount_percentage} indirim\n\n"
        message += f"{deal.title[:60]}...\n"

        # Add Human-like Comment (Shortened)
        try:
            comment = CommentGenerator.generate_comment(deal)
            # Source is now clean
            if comment:
                message += f"Editorun Notu: {comment}\n\n"
        except Exception:
            message += "\n"

        # Satıcı bilgileri (compact)
        if deal.is_amazon_fulfilled or (deal.seller_name and "amazon" in deal.seller_name.lower()):
            message += f"Satici: Amazon Turkiye\n"
        elif deal.seller_name:
            message += f"Satici: {deal.seller_name[:20]}\n"
            if deal.seller_feedback_rating:
                message += f"Satici Puani: {deal.seller_feedback_rating:.1f}"
                if deal.seller_feedback_count:
                    message += f" ({deal.seller_feedback_count})\n"
                else:
                    message += "\n"

        # Ürün değerlendirmesi (compact)
        if deal.rating:
            message += f"Urun Puani: {deal.rating:.1f}/5"
            if deal.review_count:
                message += f" ({deal.review_count} yorum)\n"
            else:
                message += "\n"

        # Kargo bilgileri (compact)
        shipping_parts = []
        if deal.is_prime_eligible:
            shipping_parts.append("Prime")
        if deal.is_free_shipping:
            shipping_parts.append("Bedava Kargo")
        if shipping_parts:
            message += " | ".join(shipping_parts) + "\n"

        # Stok durumu (sadece kritik)
        if deal.availability_message and (
            "kaldı" in deal.availability_message.lower() or "son" in deal.availability_message.lower()
        ):
            import re

            numbers = re.findall(r"\d+", deal.availability_message)
            if numbers:
                message += f"SON {numbers[0]} ADET!\n"
            else:
                message += f"Stok azaliyor!\n"

        # Fiyat bilgileri
        message += (
            f"\nFIYAT\n"
            f"Eski: {deal.original_price:.0f} TRY\n"
            f"Yeni: {deal.current_price:.0f} TRY\n"
            f"Tasarruf: {(deal.original_price - deal.current_price):.0f} TRY\n\n"
            f"{short_url}\n\n"
            f"#Amazon #işbirliği"
        )

        return message[:280]  # Twitter character limit

    def post_deal(self, deal: DealInfo) -> dict | None:
        """
        Post a deal to Twitter with product image using API v2.

        Args:
            deal: Deal information to post

        Returns:
            Dict with tweet_id if successful, None otherwise
        """
        try:
            message = self.format_deal_message(deal)

            # Try to download and upload image if URL is available
            media_id = None
            if deal.image_url:
                try:
                    logger.info(f"Downloading image from: {deal.image_url}")
                    response = requests.get(deal.image_url, timeout=10)
                    if response.status_code == 200:
                        image_data = BytesIO(response.content)
                        image_data.name = "image.jpg"
                        # Upload media using v1.1 API (still supported for media)
                        media = self.api.media_upload(filename="image.jpg", file=image_data)
                        media_id = media.media_id
                        logger.info(f"Image uploaded successfully. Media ID: {media_id}")
                    else:
                        logger.warning(f"Failed to download image: HTTP {response.status_code}")
                except Exception as img_error:
                    logger.warning(f"Could not process image: {img_error}. Posting without image.")

            # Post tweet using API v2 (Free tier supported)
            if media_id:
                tweet = self.client.create_tweet(text=message, media_ids=[str(media_id)])
                logger.info(f"Deal posted to Twitter with image. Tweet ID: {tweet.data['id']}")
            else:
                tweet = self.client.create_tweet(text=message)
                logger.info(f"Deal posted to Twitter without image. Tweet ID: {tweet.data['id']}")

            return {"tweet_id": str(tweet.data["id"])}

        except tweepy.TweepyException as e:
            logger.error(f"Failed to post deal to Twitter: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error posting to Twitter: {e}")
            return None

    def post_deal_with_image(self, deal: DealInfo, image_path: str | None = None) -> bool:
        """
        Post a deal to Twitter with an image using API v2.

        Args:
            deal: Deal information to post
            image_path: Optional path to image file

        Returns:
            True if successful, False otherwise
        """
        try:
            message = self.format_deal_message(deal)

            if image_path:
                # Upload media using v1.1 API
                media = self.api.media_upload(image_path)
                # Post tweet with media using v2 API
                tweet = self.client.create_tweet(text=message, media_ids=[str(media.media_id)])
            else:
                # Post without media using v2 API
                tweet = self.client.create_tweet(text=message)

            logger.info(f"Deal posted to Twitter successfully. Tweet ID: {tweet.data['id']}")
            return True

        except tweepy.TweepyException as e:
            logger.error(f"Failed to post deal with image to Twitter: {e}")
            return False
        except Exception as e:
            logger.error(f"Unexpected error posting to Twitter: {e}")
            return False



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.item import Item  # noqa: F401,E501
from paapi5_python_sdk.models.search_refinements import SearchRefinements  # noqa: F401,E501


class SearchResult(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "total_result_count": "int",
        "search_url": "str",
        "items": "list[Item]",
        "search_refinements": "SearchRefinements",
    }

    attribute_map = {
        "total_result_count": "TotalResultCount",
        "search_url": "SearchURL",
        "items": "Items",
        "search_refinements": "SearchRefinements",
    }

    def __init__(
        self,
        total_result_count=None,
        search_url=None,
        items=None,
        search_refinements=None,
    ):  # noqa: E501
        """SearchResult - a model defined in Swagger"""  # noqa: E501

        self._total_result_count = None
        self._search_url = None
        self._items = None
        self._search_refinements = None
        self.discriminator = None

        if total_result_count is not None:
            self.total_result_count = total_result_count
        if search_url is not None:
            self.search_url = search_url
        if items is not None:
            self.items = items
        if search_refinements is not None:
            self.search_refinements = search_refinements

    @property
    def total_result_count(self):
        """Gets the total_result_count of this SearchResult.  # noqa: E501


        :return: The total_result_count of this SearchResult.  # noqa: E501
        :rtype: int
        """
        return self._total_result_count

    @total_result_count.setter
    def total_result_count(self, total_result_count):
        """Sets the total_result_count of this SearchResult.


        :param total_result_count: The total_result_count of this SearchResult.  # noqa: E501
        :type: int
        """

        self._total_result_count = total_result_count

    @property
    def search_url(self):
        """Gets the search_url of this SearchResult.  # noqa: E501


        :return: The search_url of this SearchResult.  # noqa: E501
        :rtype: str
        """
        return self._search_url

    @search_url.setter
    def search_url(self, search_url):
        """Sets the search_url of this SearchResult.


        :param search_url: The search_url of this SearchResult.  # noqa: E501
        :type: str
        """

        self._search_url = search_url

    @property
    def items(self):
        """Gets the items of this SearchResult.  # noqa: E501


        :return: The items of this SearchResult.  # noqa: E501
        :rtype: list[Item]
        """
        return self._items

    @items.setter
    def items(self, items):
        """Sets the items of this SearchResult.


        :param items: The items of this SearchResult.  # noqa: E501
        :type: list[Item]
        """

        self._items = items

    @property
    def search_refinements(self):
        """Gets the search_refinements of this SearchResult.  # noqa: E501


        :return: The search_refinements of this SearchResult.  # noqa: E501
        :rtype: SearchRefinements
        """
        return self._search_refinements

    @search_refinements.setter
    def search_refinements(self, search_refinements):
        """Sets the search_refinements of this SearchResult.


        :param search_refinements: The search_refinements of this SearchResult.  # noqa: E501
        :type: SearchRefinements
        """

        self._search_refinements = search_refinements

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(SearchResult, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SearchResult):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Deal detection engine - rules-based deal identification using price history."""

import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import UTC, datetime, timedelta
from enum import Enum
from typing import Optional

from sqlalchemy import and_, func
from sqlalchemy.orm import Session

from opportunity_core.models.database import AlertSent, PriceHistory, PriceStatistics, Product

logger = logging.getLogger(__name__)


class DealType(Enum):
    """Types of deals the system can detect."""

    DROP_24H = "DROP_24H"  # Price dropped in last 24 hours
    NEW_LOW_7D = "NEW_LOW_7D"  # New 7-day low
    NEW_LOW_30D = "NEW_LOW_30D"  # New 30-day low
    ALL_TIME_LOW = "ALL_TIME_LOW"  # All-time lowest price
    FIRST_TIME_DEAL = "FIRST_TIME_DEAL"  # Product first discovered with good price
    SIGNIFICANT_DROP = "SIGNIFICANT_DROP"  # Large absolute price drop
    VOLATILE_DROP = "VOLATILE_DROP"  # Drop during high volatility period


@dataclass
class DetectedDeal:
    """Represents a detected deal with all context."""

    asin: str
    deal_type: DealType
    current_price: float
    previous_price: Optional[float]
    discount_amount: Optional[float]
    discount_percentage: Optional[float]
    currency: str
    product: Product
    statistics: PriceStatistics
    confidence_score: float  # 0.0-1.0, how confident we are this is a real deal
    reason: str  # Human-readable explanation
    lowest_price_30d: Optional[float] = None
    lowest_price_90d: Optional[float] = None
    lowest_price_180d: Optional[float] = None


class DealRule(ABC):
    """Abstract base class for deal detection rules."""

    @abstractmethod
    def evaluate(
        self, product: Product, current_price: PriceHistory, stats: PriceStatistics, db_session: Session
    ) -> Optional[DetectedDeal]:
        """
        Evaluate if this rule detects a deal.

        Args:
            product: Product being evaluated
            current_price: Most recent price record
            stats: Price statistics
            db_session: Database session

        Returns:
            DetectedDeal if rule triggers, None otherwise
        """
        pass

    @abstractmethod
    def get_name(self) -> str:
        """Return rule name for logging."""
        pass


class Drop24HRule(DealRule):
    """Detects price drops within last 24 hours."""

    def __init__(self, min_drop_percentage: float = 10.0, min_drop_amount: float = 10.0):
        """
        Initialize rule.

        Args:
            min_drop_percentage: Minimum percentage drop required (e.g., 10 = 10%)
            min_drop_amount: Minimum absolute price drop in currency units
        """
        self.min_drop_percentage = min_drop_percentage
        self.min_drop_amount = min_drop_amount

    def get_name(self) -> str:
        return f"24h Drop (≥{self.min_drop_percentage}% or ≥{self.min_drop_amount})"

    def evaluate(
        self, product: Product, current_price: PriceHistory, stats: PriceStatistics, db_session: Session
    ) -> Optional[DetectedDeal]:
        """Check for significant 24-hour price drop."""
        if not stats.price_change_24h or not stats.price_change_24h_pct:
            return None

        # Price must have dropped (negative change)
        if stats.price_change_24h >= 0:
            return None

        drop_amount = abs(float(stats.price_change_24h))
        drop_percentage = abs(stats.price_change_24h_pct)

        # Check thresholds
        if drop_percentage >= self.min_drop_percentage or drop_amount >= self.min_drop_amount:
            # Get previous price
            prev_price = float(current_price.price_amount) + drop_amount

            return DetectedDeal(
                asin=product.asin,
                deal_type=DealType.DROP_24H,
                current_price=float(current_price.price_amount),
                previous_price=prev_price,
                discount_amount=drop_amount,
                discount_percentage=drop_percentage,
                currency=current_price.currency,
                product=product,
                statistics=stats,
                confidence_score=min(drop_percentage / 50.0, 1.0),  # Higher % = higher confidence
                reason=f"Price dropped {drop_percentage:.1f}% ({drop_amount:.2f} {current_price.currency}) in last 24h",
            )

        return None


class NewLow30DRule(DealRule):
    """Detects new 30-day low prices."""

    def __init__(self, threshold_percentage: float = 5.0):
        """
        Initialize rule.

        Args:
            threshold_percentage: How close to minimum (e.g., 5 = within 5% of 30-day min)
        """
        self.threshold_percentage = threshold_percentage

    def get_name(self) -> str:
        return f"30-day Low (within {self.threshold_percentage}%)"

    def evaluate(
        self, product: Product, current_price: PriceHistory, stats: PriceStatistics, db_session: Session
    ) -> Optional[DetectedDeal]:
        """Check if current price is near or at 30-day minimum."""
        if not stats.min_price_30d:
            return None

        current = float(current_price.price_amount)
        min_30d = float(stats.min_price_30d)

        # Calculate how close we are to the minimum
        if current <= min_30d * (1 + self.threshold_percentage / 100):
            discount_from_avg = None
            discount_pct = None

            if stats.avg_price_30d:
                avg_30d = float(stats.avg_price_30d)
                discount_from_avg = avg_30d - current
                discount_pct = (discount_from_avg / avg_30d) * 100

            # Only report if there is a meaningful discount vs 30-day average
            if discount_pct and discount_pct >= 1.0:
                return DetectedDeal(
                    asin=product.asin,
                    deal_type=DealType.NEW_LOW_30D,
                    current_price=current,
                    previous_price=float(stats.avg_price_30d) if stats.avg_price_30d else None,
                    discount_amount=discount_from_avg,
                    discount_percentage=discount_pct,
                    currency=current_price.currency,
                    product=product,
                    statistics=stats,
                    confidence_score=0.8 if current == min_30d else 0.6,
                    reason=f"At 30-day low: {current:.2f} {current_price.currency} (-{discount_pct:.1f}% vs 30d avg)",
                )

        return None


class AllTimeLowRule(DealRule):
    """Detects all-time lowest prices."""

    def get_name(self) -> str:
        return "All-Time Low"

    def evaluate(
        self, product: Product, current_price: PriceHistory, stats: PriceStatistics, db_session: Session
    ) -> Optional[DetectedDeal]:
        """Check if current price is all-time lowest."""
        if not stats.min_price_all_time:
            return None

        current = float(current_price.price_amount)
        all_time_min = float(stats.min_price_all_time)

        # Must be exactly at all-time low
        if abs(current - all_time_min) < 0.01:  # Within 0.01 for floating point comparison
            discount_from_avg = None
            discount_pct = None

            if stats.avg_price_30d:
                avg = float(stats.avg_price_30d)
                discount_from_avg = avg - current
                discount_pct = (discount_from_avg / avg) * 100

            # 1. If we have a significant historical discount, it's a deal
            # Refined Rule: Must be at least 10% discount AND 50 TL savings vs 30-day average
            if discount_pct is not None and discount_pct >= 10.0 and discount_from_avg >= 50.0:
                return DetectedDeal(
                    asin=product.asin,
                    deal_type=DealType.ALL_TIME_LOW,
                    current_price=current,
                    previous_price=float(stats.avg_price_30d) if stats.avg_price_30d else None,
                    discount_amount=discount_from_avg,
                    discount_percentage=discount_pct,
                    currency=current_price.currency,
                    product=product,
                    statistics=stats,
                    confidence_score=1.0,  # Highest confidence
                    reason=f"🔥 ALL-TIME LOW: {current:.2f} {current_price.currency} (-{discount_pct:.0f}% vs 30d avg)",
                )

            # 2. If no historical discount (e.g., new product), check Amazon's official savings
            # This prevents 0% discount notifications for new products
            # Increased threshold to 15% to ensure only significant deals are flagged for new items
            # Also enforce 50 TL minimum savings to avoid low-value spam
            if (
                current_price.savings_percentage
                and current_price.savings_percentage >= 15
                and current_price.savings_amount
                and float(current_price.savings_amount) >= 50.0
            ):
                return DetectedDeal(
                    asin=product.asin,
                    deal_type=DealType.ALL_TIME_LOW,
                    current_price=current,
                    previous_price=float(current_price.list_price) if current_price.list_price else None,
                    discount_amount=float(current_price.savings_amount) if current_price.savings_amount else None,
                    discount_percentage=current_price.savings_percentage,
                    currency=current_price.currency,
                    product=product,
                    statistics=stats,
                    confidence_score=0.9,
                    reason=f"🔥 ALL-TIME LOW & Amazon Deal: -{current_price.savings_percentage}% (List: {current_price.list_price})",
                )

        return None


class SignificantDropRule(DealRule):
    """Detects large absolute price drops regardless of timeframe."""

    def __init__(self, min_drop_amount: float = 50.0, lookback_days: int = 7):
        """
        Initialize rule.

        Args:
            min_drop_amount: Minimum absolute price drop
            lookback_days: Days to look back for comparison
        """
        self.min_drop_amount = min_drop_amount
        self.lookback_days = lookback_days

    def get_name(self) -> str:
        return f"Significant Drop (≥{self.min_drop_amount} in {self.lookback_days}d)"

    def evaluate(
        self, product: Product, current_price: PriceHistory, stats: PriceStatistics, db_session: Session
    ) -> Optional[DetectedDeal]:
        """Check for large absolute price drops."""
        # Get price from N days ago
        lookback_date = datetime.now(UTC) - timedelta(days=self.lookback_days)

        old_price_record = (
            db_session.query(PriceHistory)
            .filter(
                and_(
                    PriceHistory.asin == product.asin,
                    PriceHistory.captured_at <= lookback_date,
                )
            )
            .order_by(PriceHistory.captured_at.desc())
            .first()
        )

        if not old_price_record:
            return None

        current = float(current_price.price_amount)
        old_price = float(old_price_record.price_amount)
        drop = old_price - current

        if drop >= self.min_drop_amount:
            drop_pct = (drop / old_price) * 100

            return DetectedDeal(
                asin=product.asin,
                deal_type=DealType.SIGNIFICANT_DROP,
                current_price=current,
                previous_price=old_price,
                discount_amount=drop,
                discount_percentage=drop_pct,
                currency=current_price.currency,
                product=product,
                statistics=stats,
                confidence_score=min(drop / 100.0, 1.0),
                reason=f"Significant drop: {drop:.2f} {current_price.currency} ({drop_pct:.1f}%) in {self.lookback_days} days",
            )

        return None


class DealDetectionEngine:
    """Main deal detection engine that applies rules and manages cooldowns."""

    def __init__(self, db_session: Session, cooldown_hours: int = 24):
        """
        Initialize deal detection engine.

        Args:
            db_session: Database session
            cooldown_hours: Hours to wait before re-alerting same ASIN
        """
        self.db_session = db_session
        self.cooldown_hours = cooldown_hours

        # Configure detection rules (customize as needed)
        self.rules = [
            AllTimeLowRule(),  # Highest priority
            Drop24HRule(min_drop_percentage=20.0, min_drop_amount=100.0),  # Aggressive: 20% or 100TL drop
            NewLow30DRule(threshold_percentage=3.0),  # Near 30-day minimum
            SignificantDropRule(min_drop_amount=100.0, lookback_days=7),  # Large drops: 100TL
        ]

        logger.info(f"Deal detection engine initialized with {len(self.rules)} rules")

    def detect_deals(self, limit: Optional[int] = None, target_asins: Optional[list[str]] = None):
        """
        Scan all active products and detect deals.
        Yields detected deals one by one.

        Args:
            limit: Maximum number of products to check (None = all)
            target_asins: Optional list of ASINs to check (None = all active)

        Yields:
            DetectedDeal objects
        """
        # Get products with recent price updates
        query = self.db_session.query(Product).filter(Product.is_active == True)

        # Filter by specific ASINs if provided
        if target_asins:
            query = query.filter(Product.asin.in_(target_asins))

        query = query.order_by(Product.check_priority, Product.last_seen_at.desc())

        if limit:
            query = query.limit(limit)

        products = query.all()

        logger.info(f"🔍 Scanning {len(products)} products for deals...")

        count = 0
        for product in products:
            deal = self._evaluate_product(product)
            if deal:
                count += 1
                yield deal

        logger.info(f"✅ Found {count} deals")

    def _evaluate_product(self, product: Product) -> Optional[DetectedDeal]:
        """
        Evaluate a single product against all rules.

        Args:
            product: Product to evaluate

        Returns:
            Highest confidence deal if detected, None otherwise
        """
        # Get latest price
        latest_price = (
            self.db_session.query(PriceHistory)
            .filter(PriceHistory.asin == product.asin)
            .order_by(PriceHistory.captured_at.desc())
            .first()
        )

        if not latest_price:
            return None

        # Get statistics
        stats = self.db_session.query(PriceStatistics).filter_by(asin=product.asin).first()

        if not stats:
            return None

        # Check cooldown (pass current price to check for stale deals)
        current_price_val = float(latest_price.price_amount)
        if self._is_in_cooldown(product.asin, current_price_val):
            logger.debug(f"  [{product.asin}] In cooldown or stale deal, skipping")
            return None

        # Apply all rules and pick best deal
        detected_deals = []
        for rule in self.rules:
            deal = rule.evaluate(product, latest_price, stats, self.db_session)
            if deal:
                logger.debug(f"  [{product.asin}] {rule.get_name()} triggered: {deal.reason}")
                detected_deals.append(deal)

        # Return highest confidence deal
        if detected_deals:
            best_deal = max(detected_deals, key=lambda d: d.confidence_score)

            # Enforce minimum thresholds (20% discount AND 100 TL savings)
            # Unless it's an ALL_TIME_LOW or FIRST_TIME_DEAL which might be special
            is_special_deal = best_deal.deal_type in [DealType.ALL_TIME_LOW, DealType.FIRST_TIME_DEAL]

            # Global Filter: Even for special deals, require minimum 100 TL savings or exceptionally high discount
            # Exception: If item is cheap (<500 TL) but has huge discount (>40%), allow it (e.g. 200 TL savings on 500 TL item)

            savings = best_deal.discount_amount or 0
            pct = best_deal.discount_percentage or 0

            # Strict Filter:
            # 1. Savings < 100 TL -> REJECT (unless >40% discount)
            # 2. Discount < 20% -> REJECT (unless All Time Low)

            if savings < 100.0:
                if pct < 40.0:
                    logger.debug(f"  [{product.asin}] Savings too low ({savings} < 100 TL), skipping")
                    return None

            if pct < 20.0 and not is_special_deal:
                logger.debug(f"  [{product.asin}] Discount too low ({pct}% < 20%), skipping")
                return None

            # Calculate historical context
            self._add_historical_context(best_deal)

            logger.info(
                f"  ✅ [{product.asin}] Deal detected: {best_deal.deal_type.value} (confidence: {best_deal.confidence_score:.2f})"
            )
            return best_deal

        return None

    def _add_historical_context(self, deal: DetectedDeal):
        """Calculate and add historical low prices to the deal object."""
        try:
            now = datetime.now(UTC)

            # Helper to get min price for a window
            def get_min_price(days: int) -> Optional[float]:
                cutoff = now - timedelta(days=days)
                min_price = (
                    self.db_session.query(func.min(PriceHistory.price_amount))
                    .filter(and_(PriceHistory.asin == deal.asin, PriceHistory.captured_at >= cutoff))
                    .scalar()
                )
                return float(min_price) if min_price else None

            deal.lowest_price_30d = get_min_price(30)
            deal.lowest_price_90d = get_min_price(90)
            deal.lowest_price_180d = get_min_price(180)

        except Exception as e:
            logger.error(f"Error calculating historical context for {deal.asin}: {e}")

    def _is_in_cooldown(self, asin: str, current_price: float) -> bool:
        """
        Check if ASIN should be silenced (cooldown or stale deal).

        Args:
            asin: Product ASIN
            current_price: Current price of the product

        Returns:
            True if should be silenced, False otherwise
        """
        # Get the very last alert sent for this ASIN
        last_alert = (
            self.db_session.query(AlertSent).filter(AlertSent.asin == asin).order_by(AlertSent.sent_at.desc()).first()
        )

        if not last_alert:
            return False  # Never alerted before

        now = datetime.now(UTC)
        time_since_alert = now - last_alert.sent_at.replace(tzinfo=UTC)

        # Check if price dropped significantly since last alert
        last_price = float(last_alert.price_amount)
        is_further_drop = current_price < last_price * 0.99

        # 1. Cooldown Check: Within 24 hours
        if time_since_alert < timedelta(hours=self.cooldown_hours):
            # If price dropped further, allow it!
            if is_further_drop:
                return False
            # Otherwise, silence it
            return True

        # 2. Stale Deal Check: If >24h passed
        # If price is same or higher than last alert, don't spam
        if is_further_drop:
            return False  # Price dropped further! Alert again.

        # Otherwise, it's the same old deal. Silence it.
        return True

    def check_platform_rate_limit(self, platform: str, limit: int, window_hours: int = 24) -> bool:
        """
        Check if the platform has reached its rate limit.

        Args:
            platform: Platform name (e.g., 'twitter')
            limit: Maximum number of alerts allowed in the window
            window_hours: Time window in hours

        Returns:
            True if limit reached (should block), False otherwise
        """
        cutoff = datetime.now(UTC) - timedelta(hours=window_hours)

        count = (
            self.db_session.query(AlertSent)
            .filter(
                and_(
                    AlertSent.platform == platform,
                    AlertSent.sent_at >= cutoff,
                )
            )
            .count()
        )

        return count >= limit

    def record_alert(self, deal: DetectedDeal, platform: str, message_id: Optional[str] = None) -> AlertSent:
        """
        Record that an alert was sent for a deal.

        Args:
            deal: The detected deal
            platform: Platform where alert was sent (telegram, twitter)
            message_id: Platform-specific message ID

        Returns:
            AlertSent record
        """
        cooldown_until = datetime.now(UTC) + timedelta(hours=self.cooldown_hours)

        alert = AlertSent(
            asin=deal.asin,
            alert_type=deal.deal_type.value,
            price_amount=deal.current_price,
            previous_price=deal.previous_price,
            discount_percentage=int(deal.discount_percentage) if deal.discount_percentage else None,
            sent_at=datetime.now(UTC),
            platform=platform,
            message_id=message_id,
            cooldown_until=cooldown_until,
        )

        self.db_session.add(alert)
        self.db_session.commit()

        logger.info(f"  📝 Alert recorded for [{deal.asin}] on {platform} (cooldown until {cooldown_until})")

        return alert



import logging
import requests
import time
from dataclasses import dataclass
from datetime import datetime, timezone
from io import BytesIO

from opportunity_core.utils.comment_generator import CommentGenerator

logger = logging.getLogger(__name__)

# ... (existing imports)


@dataclass
class DealInfo:
    """Data class for deal information."""

    title: str
    current_price: float
    original_price: float
    discount_percentage: int
    currency: str
    asin: str
    detail_page_url: str
    image_url: str | None = None
    category: str | None = None  # Product category
    rating: float | None = None  # Product rating
    review_count: int | None = None  # Number of reviews
    seller_name: str | None = None  # Merchant/seller name
    seller_feedback_count: int | None = None  # Seller feedback count
    seller_feedback_rating: float | None = None  # Seller feedback rating
    is_amazon_fulfilled: bool = False  # Amazon satıcı mı?
    availability_message: str | None = None  # Availability message
    stock_quantity: int | None = None  # Available stock
    is_prime_eligible: bool = False  # Prime üyelerine özel
    is_free_shipping: bool = False  # Ücretsiz kargo
    lowest_price_30d: float | None = None
    lowest_price_90d: float | None = None
    lowest_price_180d: float | None = None
    first_discovered_at: datetime | None = None
    deal_tag: str = "FIRSAT"  # "FIRSAT" or "FİYAT DÜŞTÜ"


class TelegramNotifier:
    """Handles posting deal notifications to Telegram channel."""

    def __init__(self, bot_token: str, channel_id: str, environment: str = "production"):
        """
        Initialize Telegram Bot.

        Args:
            bot_token: Telegram Bot token from @BotFather
            channel_id: Telegram channel ID (e.g., @channelname or -100123456789)
            environment: Environment mode ('production' or 'development')
        """
        self.bot_token = bot_token
        self.channel_id = channel_id
        self.environment = environment
        self.api_url = f"https://api.telegram.org/bot{bot_token}"

        # Verify bot connection
        try:
            response = requests.get(f"{self.api_url}/getMe", timeout=10)
            if response.status_code == 200:
                data = response.json()
                if data.get("ok"):
                    username = data["result"]["username"]
                    env_label = "🧪 TEST" if environment == "development" else "🚀 PRODUCTION"
                    logger.info(f"Connected to Telegram bot: @{username}")
                    logger.info(f"Environment: {env_label}")
                    logger.info(f"Target channel: {channel_id}")
                    logger.info("Telegram bot initialized successfully")
                else:
                    raise Exception(f"Bot verification failed: {data}")
            else:
                raise Exception(f"HTTP {response.status_code}: {response.text}")
        except Exception as e:
            logger.error(f"Telegram bot initialization failed: {e}")
            raise

    def shorten_url(self, url: str) -> str:
        """
        Shorten URL using TinyURL service.

        Args:
            url: Long URL to shorten

        Returns:
            Shortened URL or original if service fails
        """
        try:
            response = requests.get(f"http://tinyurl.com/api-create.php?url={url}", timeout=5)
            if response.status_code == 200:
                shortened = response.text
                logger.info(f"URL shortened: {url[:50]}... -> {shortened}")
                return shortened
        except Exception as e:
            logger.warning(f"URL shortening failed: {e}. Using original URL.")
        return url

    def format_deal_message(self, deal: DealInfo) -> str:
        """
        Format deal information into a Telegram message.
        Compliant with Telegram Ads guidelines (no excessive caps/emojis).

        Args:
            deal: Deal information

        Returns:
            Formatted message
        """
        # Shorten the affiliate link
        short_url = self.shorten_url(deal.detail_page_url)

        # 1. Header (Clean, no caps lock)
        # "Fırsat %45 İndirim" or "Fiyat Düştü %10 Kazanç"
        if deal.deal_tag == "FİYAT DÜŞTÜ":
            header = f"Fiyat Düştü %{deal.discount_percentage} Kazanç"
        else:
            header = f"Fırsat %{deal.discount_percentage} İndirim"

        message = f"<b>{header}</b>\n\n"

        # 2. Product Title (Clean)
        message += f"{deal.title}\n\n"

        # 3. Editor's Note (Human-like comment)
        try:
            comment = CommentGenerator.generate_comment(deal)

            # Source is now clean, no need for extra processing
            if comment:
                message += f"<b>Editörün Notu:</b> {comment}\n\n"
        except Exception as e:
            logger.warning(f"Failed to generate comment: {e}")

        # 4. Price Analysis (Bulleted list, professional)
        message += f"<b>Fiyat Analizi:</b>\n"
        # Add strikethrough to old price
        message += f"• Eski Fiyat: <s>{deal.original_price:,.2f} {deal.currency}</s>\n".replace(",", ".")
        message += f"• Yeni Fiyat: {deal.current_price:,.2f} {deal.currency}\n".replace(",", ".")

        savings = deal.original_price - deal.current_price
        message += f"• Kazanç: {savings:,.2f} {deal.currency}\n".replace(",", ".")

        # Historical Context (Clean text, no "WARNING" emojis)
        days_tracked = 0
        if deal.first_discovered_at:
            now = datetime.now(timezone.utc)
            discovered = deal.first_discovered_at
            if discovered.tzinfo is None:
                discovered = discovered.replace(tzinfo=timezone.utc)
            days_tracked = (now - discovered).days

        if deal.lowest_price_180d and deal.current_price <= deal.lowest_price_180d and days_tracked >= 180:
            message += f"• Not: Son 6 ayın en düşük fiyatı.\n"
        elif deal.lowest_price_90d and deal.current_price <= deal.lowest_price_90d and days_tracked >= 90:
            message += f"• Not: Son 3 ayın en düşük fiyatı.\n"
        elif deal.lowest_price_30d and deal.current_price <= deal.lowest_price_30d and days_tracked >= 30:
            message += f"• Not: Son 1 ayın en düşük fiyatı.\n"

        message += "\n"

        # 5. Footer / CTA (Simple link)
        message += f"🔗 <a href='{short_url}'>Fırsata Git</a>\n\n"

        # 6. Tags
        message += "#Amazon #işbirliği"

        return message

    def post_deal(self, deal: DealInfo) -> dict | None:
        """
        Post a deal to Telegram channel.

        Args:
            deal: Deal information to post

        Returns:
            Dict with message_id if successful, None otherwise
        """
        message = self.format_deal_message(deal)
        max_retries = 3

        for attempt in range(max_retries):
            try:
                if deal.image_url:
                    # Download image
                    logger.info(f"Downloading image from: {deal.image_url}")
                    img_response = requests.get(deal.image_url, timeout=20)  # Increased timeout for image download

                    if img_response.status_code == 200:
                        # Send photo with caption using multipart/form-data
                        files = {"photo": ("product.jpg", BytesIO(img_response.content), "image/jpeg")}
                        data = {"chat_id": self.channel_id, "caption": message, "parse_mode": "HTML"}

                        # Increased timeout to 60s for upload
                        response = requests.post(f"{self.api_url}/sendPhoto", files=files, data=data, timeout=60)

                        if response.status_code == 200 and response.json().get("ok"):
                            logger.info("Deal posted to Telegram with image")
                            result = response.json()["result"]
                            return {"message_id": str(result["message_id"])}
                        else:
                            error_msg = response.json().get("description", response.text)
                            logger.error(f"Failed to post deal to Telegram: {error_msg}")
                            # Don't retry client errors (4xx) unless it's 429
                            if 400 <= response.status_code < 500 and response.status_code != 429:
                                return None
                    else:
                        logger.warning(f"Failed to download image: HTTP {img_response.status_code}")
                        # Fallback to text message immediately if image fails
                        # but continue in the same logic flow

                # Send text message if no image or image download failed
                data = {
                    "chat_id": self.channel_id,
                    "text": message,
                    "parse_mode": "HTML",
                    "disable_web_page_preview": False,
                }

                response = requests.post(f"{self.api_url}/sendMessage", json=data, timeout=60)

                if response.status_code == 200 and response.json().get("ok"):
                    logger.info("Deal posted to Telegram without image")
                    result = response.json()["result"]
                    return {"message_id": str(result["message_id"])}
                else:
                    error_msg = response.json().get("description", response.text)
                    logger.error(f"Failed to post deal to Telegram: {error_msg}")
                    if 400 <= response.status_code < 500 and response.status_code != 429:
                        return None

            except requests.exceptions.RequestException as e:
                wait_time = (attempt + 1) * 2
                logger.warning(f"Attempt {attempt + 1}/{max_retries} failed: {e}. Retrying in {wait_time}s...")
                time.sleep(wait_time)
            except Exception as e:
                logger.error(f"Unexpected error posting to Telegram: {e}")
                return None

        logger.error(f"❌ All {max_retries} attempts to post to Telegram failed.")
        return None



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.languages import Languages  # noqa: F401,E501
from paapi5_python_sdk.models.single_integer_valued_attribute import (
    SingleIntegerValuedAttribute,
)  # noqa: F401,E501
from paapi5_python_sdk.models.single_string_valued_attribute import (
    SingleStringValuedAttribute,
)  # noqa: F401,E501


class ContentInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "edition": "SingleStringValuedAttribute",
        "languages": "Languages",
        "pages_count": "SingleIntegerValuedAttribute",
        "publication_date": "SingleStringValuedAttribute",
    }

    attribute_map = {
        "edition": "Edition",
        "languages": "Languages",
        "pages_count": "PagesCount",
        "publication_date": "PublicationDate",
    }

    def __init__(self, edition=None, languages=None, pages_count=None, publication_date=None):  # noqa: E501
        """ContentInfo - a model defined in Swagger"""  # noqa: E501

        self._edition = None
        self._languages = None
        self._pages_count = None
        self._publication_date = None
        self.discriminator = None

        if edition is not None:
            self.edition = edition
        if languages is not None:
            self.languages = languages
        if pages_count is not None:
            self.pages_count = pages_count
        if publication_date is not None:
            self.publication_date = publication_date

    @property
    def edition(self):
        """Gets the edition of this ContentInfo.  # noqa: E501


        :return: The edition of this ContentInfo.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._edition

    @edition.setter
    def edition(self, edition):
        """Sets the edition of this ContentInfo.


        :param edition: The edition of this ContentInfo.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._edition = edition

    @property
    def languages(self):
        """Gets the languages of this ContentInfo.  # noqa: E501


        :return: The languages of this ContentInfo.  # noqa: E501
        :rtype: Languages
        """
        return self._languages

    @languages.setter
    def languages(self, languages):
        """Sets the languages of this ContentInfo.


        :param languages: The languages of this ContentInfo.  # noqa: E501
        :type: Languages
        """

        self._languages = languages

    @property
    def pages_count(self):
        """Gets the pages_count of this ContentInfo.  # noqa: E501


        :return: The pages_count of this ContentInfo.  # noqa: E501
        :rtype: SingleIntegerValuedAttribute
        """
        return self._pages_count

    @pages_count.setter
    def pages_count(self, pages_count):
        """Sets the pages_count of this ContentInfo.


        :param pages_count: The pages_count of this ContentInfo.  # noqa: E501
        :type: SingleIntegerValuedAttribute
        """

        self._pages_count = pages_count

    @property
    def publication_date(self):
        """Gets the publication_date of this ContentInfo.  # noqa: E501


        :return: The publication_date of this ContentInfo.  # noqa: E501
        :rtype: SingleStringValuedAttribute
        """
        return self._publication_date

    @publication_date.setter
    def publication_date(self, publication_date):
        """Sets the publication_date of this ContentInfo.


        :param publication_date: The publication_date of this ContentInfo.  # noqa: E501
        :type: SingleStringValuedAttribute
        """

        self._publication_date = publication_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(ContentInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ContentInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
